@@ -15,21 +15,14 @@ INCLUDES = """
  * Note that the result is an opaque type.
  */
 typedef STACK_OF(X509) Cryptography_STACK_OF_X509;
-typedef STACK_OF(X509_CRL) Cryptography_STACK_OF_X509_CRL;
 typedef STACK_OF(X509_REVOKED) Cryptography_STACK_OF_X509_REVOKED;
 """
 
 TYPES = """
 typedef ... Cryptography_STACK_OF_X509;
-typedef ... Cryptography_STACK_OF_X509_CRL;
 typedef ... Cryptography_STACK_OF_X509_REVOKED;
 
-typedef struct {
-    ASN1_OBJECT *algorithm;
-    ...;
-} X509_ALGOR;
-
-typedef ... X509_ATTRIBUTE;
+typedef ... X509_ALGOR;
 typedef ... X509_EXTENSION;
 typedef ... X509_EXTENSIONS;
 typedef ... X509_REQ;
@@ -37,10 +30,6 @@ typedef ... X509_REVOKED;
 typedef ... X509_CRL;
 typedef ... X509;
 
-typedef ... NETSCAPE_SPKI;
-
-typedef ... PKCS8_PRIV_KEY_INFO;
-
 typedef void (*sk_X509_EXTENSION_freefunc)(X509_EXTENSION *);
 """
 
@@ -112,14 +101,6 @@ int X509_CRL_sort(X509_CRL *);
 int i2d_X509_CRL_bio(BIO *, X509_CRL *);
 void X509_CRL_free(X509_CRL *);
 
-int NETSCAPE_SPKI_verify(NETSCAPE_SPKI *, EVP_PKEY *);
-int NETSCAPE_SPKI_sign(NETSCAPE_SPKI *, EVP_PKEY *, const EVP_MD *);
-char *NETSCAPE_SPKI_b64_encode(NETSCAPE_SPKI *);
-EVP_PKEY *NETSCAPE_SPKI_get_pubkey(NETSCAPE_SPKI *);
-int NETSCAPE_SPKI_set_pubkey(NETSCAPE_SPKI *, EVP_PKEY *);
-NETSCAPE_SPKI *NETSCAPE_SPKI_new(void);
-void NETSCAPE_SPKI_free(NETSCAPE_SPKI *);
-
 /*  ASN1 serialization */
 int i2d_X509_bio(BIO *, X509 *);
 X509 *d2i_X509_bio(BIO *, X509 **);
@@ -142,11 +123,6 @@ const char *X509_get_default_cert_file(void);
 const char *X509_get_default_cert_dir_env(void);
 const char *X509_get_default_cert_file_env(void);
 
-int i2d_RSAPrivateKey_bio(BIO *, RSA *);
-RSA *d2i_RSAPublicKey_bio(BIO *, RSA **);
-int i2d_RSAPublicKey_bio(BIO *, RSA *);
-int i2d_DSAPrivateKey_bio(BIO *, DSA *);
-
 int X509_get_ext_count(const X509 *);
 X509_EXTENSION *X509_get_ext(const X509 *, int);
 X509_NAME *X509_get_subject_name(const X509 *);
@@ -191,8 +167,6 @@ Cryptography_STACK_OF_X509_REVOKED *X509_CRL_get_REVOKED(X509_CRL *);
 int X509_CRL_set1_lastUpdate(X509_CRL *, const ASN1_TIME *);
 int X509_CRL_set1_nextUpdate(X509_CRL *, const ASN1_TIME *);
 
-int i2d_ECPrivateKey_bio(BIO *, EC_KEY *);
-
 const ASN1_INTEGER *X509_REVOKED_get0_serialNumber(const X509_REVOKED *);
 const ASN1_TIME *X509_REVOKED_get0_revocationDate(const X509_REVOKED *);
 
