@@ -2,22 +2,30 @@
 // 2.0, and the BSD License. See the LICENSE file in the root of this repository
 // for complete details.
 
-use crate::error::CryptographyResult;
-use crate::x509;
-use crate::x509::certificate::Certificate;
+use std::collections::HashMap;
+
 use cryptography_x509::common;
 use cryptography_x509::ocsp_req::CertID;
 use once_cell::sync::Lazy;
-use std::collections::HashMap;
+use pyo3::types::PyAnyMethods;
+
+use crate::backend::hashes::Hash;
+use crate::error::CryptographyResult;
+use crate::x509::certificate::Certificate;
 
 pub(crate) static ALGORITHM_PARAMETERS_TO_HASH: Lazy<
     HashMap<common::AlgorithmParameters<'_>, &str>,
 > = Lazy::new(|| {
     let mut h = HashMap::new();
+    h.insert(common::AlgorithmParameters::Sha1(None), "SHA1");
     h.insert(common::AlgorithmParameters::Sha1(Some(())), "SHA1");
+    h.insert(common::AlgorithmParameters::Sha224(None), "SHA224");
     h.insert(common::AlgorithmParameters::Sha224(Some(())), "SHA224");
+    h.insert(common::AlgorithmParameters::Sha256(None), "SHA256");
     h.insert(common::AlgorithmParameters::Sha256(Some(())), "SHA256");
+    h.insert(common::AlgorithmParameters::Sha384(None), "SHA384");
     h.insert(common::AlgorithmParameters::Sha384(Some(())), "SHA384");
+    h.insert(common::AlgorithmParameters::Sha512(None), "SHA512");
     h.insert(common::AlgorithmParameters::Sha512(Some(())), "SHA512");
     h
 });
@@ -66,32 +74,34 @@ pub(crate) static HASH_NAME_TO_ALGORITHM_IDENTIFIERS: Lazy<
 
 pub(crate) fn certid_new<'p>(
     py: pyo3::Python<'p>,
+    ka: &'p cryptography_keepalive::KeepAlive<pyo3::pybacked::PyBackedBytes>,
     cert: &'p Certificate,
     issuer: &'p Certificate,
-    hash_algorithm: &'p pyo3::PyAny,
+    hash_algorithm: &pyo3::Bound<'p, pyo3::PyAny>,
 ) -> CryptographyResult<CertID<'p>> {
-    let issuer_der = asn1::write_single(&cert.raw.borrow_value_public().tbs_cert.issuer)?;
-    let issuer_name_hash = hash_data(py, hash_algorithm, &issuer_der)?;
-    let issuer_key_hash = hash_data(
+    let issuer_der = asn1::write_single(&cert.raw.borrow_dependent().tbs_cert.issuer)?;
+    let issuer_name_hash =
+        pyo3::pybacked::PyBackedBytes::from(hash_data(py, hash_algorithm, &issuer_der)?);
+    let issuer_key_hash = pyo3::pybacked::PyBackedBytes::from(hash_data(
         py,
         hash_algorithm,
         issuer
             .raw
-            .borrow_value_public()
+            .borrow_dependent()
             .tbs_cert
             .spki
             .subject_public_key
             .as_bytes(),
-    )?;
+    )?);
 
     Ok(CertID {
-        hash_algorithm: x509::ocsp::HASH_NAME_TO_ALGORITHM_IDENTIFIERS[hash_algorithm
+        hash_algorithm: HASH_NAME_TO_ALGORITHM_IDENTIFIERS[&*hash_algorithm
             .getattr(pyo3::intern!(py, "name"))?
-            .extract::<&str>()?]
-        .clone(),
-        issuer_name_hash,
-        issuer_key_hash,
-        serial_number: cert.raw.borrow_value_public().tbs_cert.serial,
+            .extract::<pyo3::pybacked::PyBackedStr>()?]
+            .clone(),
+        issuer_name_hash: ka.add(issuer_name_hash),
+        issuer_key_hash: ka.add(issuer_key_hash),
+        serial_number: cert.raw.borrow_dependent().tbs_cert.serial,
     })
 }
 
@@ -100,13 +110,13 @@ pub(crate) fn certid_new_from_hash<'p>(
     issuer_name_hash: &'p [u8],
     issuer_key_hash: &'p [u8],
     serial_number: asn1::BigInt<'p>,
-    hash_algorithm: &'p pyo3::PyAny,
+    hash_algorithm: pyo3::Bound<'p, pyo3::PyAny>,
 ) -> CryptographyResult<CertID<'p>> {
+    let hash_name = hash_algorithm
+        .getattr(pyo3::intern!(py, "name"))?
+        .extract::<pyo3::pybacked::PyBackedStr>()?;
     Ok(CertID {
-        hash_algorithm: x509::ocsp::HASH_NAME_TO_ALGORITHM_IDENTIFIERS[hash_algorithm
-            .getattr(pyo3::intern!(py, "name"))?
-            .extract::<&str>()?]
-        .clone(),
+        hash_algorithm: HASH_NAME_TO_ALGORITHM_IDENTIFIERS[&*hash_name].clone(),
         issuer_name_hash,
         issuer_key_hash,
         serial_number,
@@ -115,13 +125,10 @@ pub(crate) fn certid_new_from_hash<'p>(
 
 pub(crate) fn hash_data<'p>(
     py: pyo3::Python<'p>,
-    py_hash_alg: &'p pyo3::PyAny,
+    py_hash_alg: &pyo3::Bound<'p, pyo3::PyAny>,
     data: &[u8],
-) -> pyo3::PyResult<&'p [u8]> {
-    let hash = py
-        .import(pyo3::intern!(py, "cryptography.hazmat.primitives.hashes"))?
-        .getattr(pyo3::intern!(py, "Hash"))?
-        .call1((py_hash_alg,))?;
-    hash.call_method1(pyo3::intern!(py, "update"), (data,))?;
-    hash.call_method0(pyo3::intern!(py, "finalize"))?.extract()
+) -> pyo3::PyResult<pyo3::Bound<'p, pyo3::types::PyBytes>> {
+    let mut h = Hash::new(py, py_hash_alg, None)?;
+    h.update_bytes(data)?;
+    Ok(h.finalize(py)?)
 }
