@@ -2,12 +2,16 @@
 // 2.0, and the BSD License. See the LICENSE file in the root of this repository
 // for complete details.
 
-use crate::asn1::oid_to_py_oid;
-use crate::error::{CryptographyError, CryptographyResult};
-use crate::exceptions;
+use std::collections::HashMap;
+
 use cryptography_x509::{common, oid};
 use once_cell::sync::Lazy;
-use std::collections::HashMap;
+use pyo3::pybacked::PyBackedBytes;
+use pyo3::types::PyAnyMethods;
+
+use crate::asn1::oid_to_py_oid;
+use crate::error::{CryptographyError, CryptographyResult};
+use crate::{exceptions, types};
 
 // This is similar to a hashmap in ocsp.rs but contains more hash algorithms
 // that aren't allowable in OCSP
@@ -46,52 +50,19 @@ enum HashType {
     Sha3_512,
 }
 
-fn identify_key_type(py: pyo3::Python<'_>, private_key: &pyo3::PyAny) -> pyo3::PyResult<KeyType> {
-    let rsa_private_key: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.rsa"
-        ))?
-        .getattr(pyo3::intern!(py, "RSAPrivateKey"))?
-        .extract()?;
-    let dsa_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.dsa"
-        ))?
-        .getattr(pyo3::intern!(py, "DSAPrivateKey"))?
-        .extract()?;
-    let ec_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.ec"
-        ))?
-        .getattr(pyo3::intern!(py, "EllipticCurvePrivateKey"))?
-        .extract()?;
-    let ed25519_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.ed25519"
-        ))?
-        .getattr(pyo3::intern!(py, "Ed25519PrivateKey"))?
-        .extract()?;
-    let ed448_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.ed448"
-        ))?
-        .getattr(pyo3::intern!(py, "Ed448PrivateKey"))?
-        .extract()?;
-
-    if private_key.is_instance(rsa_private_key)? {
+pub(crate) fn identify_key_type(
+    py: pyo3::Python<'_>,
+    private_key: pyo3::Bound<'_, pyo3::PyAny>,
+) -> pyo3::PyResult<KeyType> {
+    if private_key.is_instance(&types::RSA_PRIVATE_KEY.get(py)?)? {
         Ok(KeyType::Rsa)
-    } else if private_key.is_instance(dsa_key_type)? {
+    } else if private_key.is_instance(&types::DSA_PRIVATE_KEY.get(py)?)? {
         Ok(KeyType::Dsa)
-    } else if private_key.is_instance(ec_key_type)? {
+    } else if private_key.is_instance(&types::ELLIPTIC_CURVE_PRIVATE_KEY.get(py)?)? {
         Ok(KeyType::Ec)
-    } else if private_key.is_instance(ed25519_key_type)? {
+    } else if private_key.is_instance(&types::ED25519_PRIVATE_KEY.get(py)?)? {
         Ok(KeyType::Ed25519)
-    } else if private_key.is_instance(ed448_key_type)? {
+    } else if private_key.is_instance(&types::ED448_PRIVATE_KEY.get(py)?)? {
         Ok(KeyType::Ed448)
     } else {
         Err(pyo3::exceptions::PyTypeError::new_err(
@@ -102,25 +73,21 @@ fn identify_key_type(py: pyo3::Python<'_>, private_key: &pyo3::PyAny) -> pyo3::P
 
 fn identify_hash_type(
     py: pyo3::Python<'_>,
-    hash_algorithm: &pyo3::PyAny,
+    hash_algorithm: pyo3::Bound<'_, pyo3::PyAny>,
 ) -> pyo3::PyResult<HashType> {
     if hash_algorithm.is_none() {
         return Ok(HashType::None);
     }
 
-    let hash_algorithm_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(py, "cryptography.hazmat.primitives.hashes"))?
-        .getattr(pyo3::intern!(py, "HashAlgorithm"))?
-        .extract()?;
-    if !hash_algorithm.is_instance(hash_algorithm_type)? {
+    if !hash_algorithm.is_instance(&types::HASH_ALGORITHM.get(py)?)? {
         return Err(pyo3::exceptions::PyTypeError::new_err(
             "Algorithm must be a registered hash algorithm.",
         ));
     }
 
-    match hash_algorithm
+    match &*hash_algorithm
         .getattr(pyo3::intern!(py, "name"))?
-        .extract()?
+        .extract::<pyo3::pybacked::PyBackedStr>()?
     {
         "sha224" => Ok(HashType::Sha224),
         "sha256" => Ok(HashType::Sha256),
@@ -131,35 +98,28 @@ fn identify_hash_type(
         "sha3-384" => Ok(HashType::Sha3_384),
         "sha3-512" => Ok(HashType::Sha3_512),
         name => Err(exceptions::UnsupportedAlgorithm::new_err(format!(
-            "Hash algorithm {:?} not supported for signatures",
-            name
+            "Hash algorithm {name:?} not supported for signatures"
         ))),
     }
 }
 
 fn compute_pss_salt_length<'p>(
     py: pyo3::Python<'p>,
-    private_key: &'p pyo3::PyAny,
-    hash_algorithm: &'p pyo3::PyAny,
-    rsa_padding: &'p pyo3::PyAny,
+    private_key: pyo3::Bound<'p, pyo3::PyAny>,
+    hash_algorithm: pyo3::Bound<'p, pyo3::PyAny>,
+    rsa_padding: pyo3::Bound<'p, pyo3::PyAny>,
 ) -> pyo3::PyResult<u16> {
-    let padding_mod = py.import(pyo3::intern!(
-        py,
-        "cryptography.hazmat.primitives.asymmetric.padding"
-    ))?;
-    let maxlen = padding_mod.getattr(pyo3::intern!(py, "_MaxLength"))?;
-    let digestlen = padding_mod.getattr(pyo3::intern!(py, "_DigestLength"))?;
     let py_saltlen = rsa_padding.getattr(pyo3::intern!(py, "_salt_length"))?;
-    if py_saltlen.is_instance(maxlen)? {
-        padding_mod
-            .getattr(pyo3::intern!(py, "calculate_max_pss_salt_length"))?
+    if py_saltlen.is_instance(&types::PADDING_MAX_LENGTH.get(py)?)? {
+        types::CALCULATE_MAX_PSS_SALT_LENGTH
+            .get(py)?
             .call1((private_key, hash_algorithm))?
             .extract::<u16>()
-    } else if py_saltlen.is_instance(digestlen)? {
+    } else if py_saltlen.is_instance(&types::PADDING_DIGEST_LENGTH.get(py)?)? {
         hash_algorithm
             .getattr(pyo3::intern!(py, "digest_size"))?
             .extract::<u16>()
-    } else if py_saltlen.is_instance(py.get_type::<pyo3::types::PyLong>())? {
+    } else if py_saltlen.is_instance_of::<pyo3::types::PyLong>() {
         py_saltlen.extract::<u16>()
     } else {
         Err(pyo3::exceptions::PyTypeError::new_err(
@@ -170,29 +130,23 @@ fn compute_pss_salt_length<'p>(
 
 pub(crate) fn compute_signature_algorithm<'p>(
     py: pyo3::Python<'p>,
-    private_key: &'p pyo3::PyAny,
-    hash_algorithm: &'p pyo3::PyAny,
-    rsa_padding: &'p pyo3::PyAny,
+    private_key: pyo3::Bound<'p, pyo3::PyAny>,
+    hash_algorithm: pyo3::Bound<'p, pyo3::PyAny>,
+    rsa_padding: pyo3::Bound<'p, pyo3::PyAny>,
 ) -> pyo3::PyResult<common::AlgorithmIdentifier<'static>> {
-    let key_type = identify_key_type(py, private_key)?;
-    let hash_type = identify_hash_type(py, hash_algorithm)?;
+    let key_type = identify_key_type(py, private_key.clone())?;
+    let hash_type = identify_hash_type(py, hash_algorithm.clone())?;
 
-    let pss_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.padding"
-        ))?
-        .getattr(pyo3::intern!(py, "PSS"))?
-        .extract()?;
     // If this is RSA-PSS we need to compute the signature algorithm from the
     // parameters provided in rsa_padding.
-    if !rsa_padding.is_none() && rsa_padding.is_instance(pss_type)? {
+    if rsa_padding.is_instance(&types::PSS.get(py)?)? {
         let hash_alg_params = identify_alg_params_for_hash_type(hash_type)?;
         let hash_algorithm_id = common::AlgorithmIdentifier {
             oid: asn1::DefinedByMarker::marker(),
             params: hash_alg_params,
         };
-        let salt_length = compute_pss_salt_length(py, private_key, hash_algorithm, rsa_padding)?;
+        let salt_length =
+            compute_pss_salt_length(py, private_key, hash_algorithm, rsa_padding.clone())?;
         let py_mgf_alg = rsa_padding
             .getattr(pyo3::intern!(py, "_mgf"))?
             .getattr(pyo3::intern!(py, "_algorithm"))?;
@@ -209,7 +163,7 @@ pub(crate) fn compute_signature_algorithm<'p>(
                     params: mgf_alg,
                 },
                 salt_length,
-                _trailer_field: 1,
+                _trailer_field: None,
             })));
 
         return Ok(common::AlgorithmIdentifier {
@@ -300,19 +254,19 @@ pub(crate) fn compute_signature_algorithm<'p>(
 
         (KeyType::Dsa, HashType::Sha224) => Ok(common::AlgorithmIdentifier {
             oid: asn1::DefinedByMarker::marker(),
-            params: common::AlgorithmParameters::DsaWithSha224,
+            params: common::AlgorithmParameters::DsaWithSha224(None),
         }),
         (KeyType::Dsa, HashType::Sha256) => Ok(common::AlgorithmIdentifier {
             oid: asn1::DefinedByMarker::marker(),
-            params: common::AlgorithmParameters::DsaWithSha256,
+            params: common::AlgorithmParameters::DsaWithSha256(None),
         }),
         (KeyType::Dsa, HashType::Sha384) => Ok(common::AlgorithmIdentifier {
             oid: asn1::DefinedByMarker::marker(),
-            params: common::AlgorithmParameters::DsaWithSha384,
+            params: common::AlgorithmParameters::DsaWithSha384(None),
         }),
         (KeyType::Dsa, HashType::Sha512) => Ok(common::AlgorithmIdentifier {
             oid: asn1::DefinedByMarker::marker(),
-            params: common::AlgorithmParameters::DsaWithSha512,
+            params: common::AlgorithmParameters::DsaWithSha512(None),
         }),
         (
             KeyType::Dsa,
@@ -328,37 +282,25 @@ pub(crate) fn compute_signature_algorithm<'p>(
 
 pub(crate) fn sign_data<'p>(
     py: pyo3::Python<'p>,
-    private_key: &'p pyo3::PyAny,
-    hash_algorithm: &'p pyo3::PyAny,
-    rsa_padding: &'p pyo3::PyAny,
+    private_key: pyo3::Bound<'p, pyo3::PyAny>,
+    hash_algorithm: pyo3::Bound<'p, pyo3::PyAny>,
+    rsa_padding: pyo3::Bound<'p, pyo3::PyAny>,
     data: &[u8],
-) -> pyo3::PyResult<&'p [u8]> {
-    let key_type = identify_key_type(py, private_key)?;
+) -> pyo3::PyResult<PyBackedBytes> {
+    let key_type = identify_key_type(py, private_key.clone())?;
 
     let signature = match key_type {
         KeyType::Ed25519 | KeyType::Ed448 => {
             private_key.call_method1(pyo3::intern!(py, "sign"), (data,))?
         }
         KeyType::Ec => {
-            let ec_mod = py.import(pyo3::intern!(
-                py,
-                "cryptography.hazmat.primitives.asymmetric.ec"
-            ))?;
-            let ecdsa = ec_mod
-                .getattr(pyo3::intern!(py, "ECDSA"))?
-                .call1((hash_algorithm,))?;
+            let ecdsa = types::ECDSA.get(py)?.call1((hash_algorithm,))?;
             private_key.call_method1(pyo3::intern!(py, "sign"), (data, ecdsa))?
         }
         KeyType::Rsa => {
             let mut padding = rsa_padding;
             if padding.is_none() {
-                let padding_mod = py.import(pyo3::intern!(
-                    py,
-                    "cryptography.hazmat.primitives.asymmetric.padding"
-                ))?;
-                padding = padding_mod
-                    .getattr(pyo3::intern!(py, "PKCS1v15"))?
-                    .call0()?;
+                padding = types::PKCS1V15.get(py)?.call0()?;
             }
             private_key.call_method1(pyo3::intern!(py, "sign"), (data, padding, hash_algorithm))?
         }
@@ -371,12 +313,12 @@ pub(crate) fn sign_data<'p>(
 
 pub(crate) fn verify_signature_with_signature_algorithm<'p>(
     py: pyo3::Python<'p>,
-    issuer_public_key: &'p pyo3::PyAny,
+    issuer_public_key: pyo3::Bound<'p, pyo3::PyAny>,
     signature_algorithm: &common::AlgorithmIdentifier<'_>,
     signature: &[u8],
     data: &[u8],
 ) -> CryptographyResult<()> {
-    let key_type = identify_public_key_type(py, issuer_public_key)?;
+    let key_type = identify_public_key_type(py, issuer_public_key.clone())?;
     let sig_key_type = identify_key_type_for_algorithm_params(&signature_algorithm.params)?;
     if key_type != sig_key_type {
         return Err(CryptographyError::from(
@@ -415,53 +357,17 @@ pub(crate) fn verify_signature_with_signature_algorithm<'p>(
 
 pub(crate) fn identify_public_key_type(
     py: pyo3::Python<'_>,
-    public_key: &pyo3::PyAny,
+    public_key: pyo3::Bound<'_, pyo3::PyAny>,
 ) -> pyo3::PyResult<KeyType> {
-    let rsa_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.rsa"
-        ))?
-        .getattr(pyo3::intern!(py, "RSAPublicKey"))?
-        .extract()?;
-    let dsa_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.dsa"
-        ))?
-        .getattr(pyo3::intern!(py, "DSAPublicKey"))?
-        .extract()?;
-    let ec_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.ec"
-        ))?
-        .getattr(pyo3::intern!(py, "EllipticCurvePublicKey"))?
-        .extract()?;
-    let ed25519_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.ed25519"
-        ))?
-        .getattr(pyo3::intern!(py, "Ed25519PublicKey"))?
-        .extract()?;
-    let ed448_key_type: &pyo3::types::PyType = py
-        .import(pyo3::intern!(
-            py,
-            "cryptography.hazmat.primitives.asymmetric.ed448"
-        ))?
-        .getattr(pyo3::intern!(py, "Ed448PublicKey"))?
-        .extract()?;
-
-    if public_key.is_instance(rsa_key_type)? {
+    if public_key.is_instance(&types::RSA_PUBLIC_KEY.get(py)?)? {
         Ok(KeyType::Rsa)
-    } else if public_key.is_instance(dsa_key_type)? {
+    } else if public_key.is_instance(&types::DSA_PUBLIC_KEY.get(py)?)? {
         Ok(KeyType::Dsa)
-    } else if public_key.is_instance(ec_key_type)? {
+    } else if public_key.is_instance(&types::ELLIPTIC_CURVE_PUBLIC_KEY.get(py)?)? {
         Ok(KeyType::Ec)
-    } else if public_key.is_instance(ed25519_key_type)? {
+    } else if public_key.is_instance(&types::ED25519_PUBLIC_KEY.get(py)?)? {
         Ok(KeyType::Ed25519)
-    } else if public_key.is_instance(ed448_key_type)? {
+    } else if public_key.is_instance(&types::ED448_PUBLIC_KEY.get(py)?)? {
         Ok(KeyType::Ed448)
     } else {
         Err(pyo3::exceptions::PyTypeError::new_err(
@@ -493,10 +399,10 @@ fn identify_key_type_for_algorithm_params(
         | common::AlgorithmParameters::EcDsaWithSha3_512 => Ok(KeyType::Ec),
         common::AlgorithmParameters::Ed25519 => Ok(KeyType::Ed25519),
         common::AlgorithmParameters::Ed448 => Ok(KeyType::Ed448),
-        common::AlgorithmParameters::DsaWithSha224
-        | common::AlgorithmParameters::DsaWithSha256
-        | common::AlgorithmParameters::DsaWithSha384
-        | common::AlgorithmParameters::DsaWithSha512 => Ok(KeyType::Dsa),
+        common::AlgorithmParameters::DsaWithSha224(..)
+        | common::AlgorithmParameters::DsaWithSha256(..)
+        | common::AlgorithmParameters::DsaWithSha384(..)
+        | common::AlgorithmParameters::DsaWithSha512(..) => Ok(KeyType::Dsa),
         _ => Err(pyo3::exceptions::PyValueError::new_err(
             "Unsupported signature algorithm",
         )),
@@ -524,10 +430,9 @@ fn identify_alg_params_for_hash_type(
 fn hash_oid_py_hash(
     py: pyo3::Python<'_>,
     oid: asn1::ObjectIdentifier,
-) -> CryptographyResult<&pyo3::PyAny> {
-    let hashes = py.import(pyo3::intern!(py, "cryptography.hazmat.primitives.hashes"))?;
+) -> CryptographyResult<pyo3::Bound<'_, pyo3::PyAny>> {
     match HASH_OIDS_TO_HASH.get(&oid) {
-        Some(alg_name) => Ok(hashes.getattr(*alg_name)?.call0()?),
+        Some(alg_name) => Ok(types::HASHES_MODULE.get(py)?.getattr(*alg_name)?.call0()?),
         None => Err(CryptographyError::from(
             exceptions::UnsupportedAlgorithm::new_err(format!(
                 "Signature algorithm OID: {} not recognized",
@@ -540,10 +445,8 @@ fn hash_oid_py_hash(
 pub(crate) fn identify_signature_hash_algorithm<'p>(
     py: pyo3::Python<'p>,
     signature_algorithm: &common::AlgorithmIdentifier<'_>,
-) -> CryptographyResult<&'p pyo3::PyAny> {
-    let sig_oids_to_hash = py
-        .import(pyo3::intern!(py, "cryptography.hazmat._oid"))?
-        .getattr(pyo3::intern!(py, "_SIG_OIDS_TO_HASH"))?;
+) -> CryptographyResult<pyo3::Bound<'p, pyo3::PyAny>> {
+    let sig_oids_to_hash = types::SIG_OIDS_TO_HASH.get(py)?;
     match &signature_algorithm.params {
         common::AlgorithmParameters::RsaPss(opt_pss) => {
             let pss = opt_pss.as_ref().ok_or_else(|| {
@@ -570,7 +473,7 @@ pub(crate) fn identify_signature_hash_algorithm<'p>(
 pub(crate) fn identify_signature_algorithm_parameters<'p>(
     py: pyo3::Python<'p>,
     signature_algorithm: &common::AlgorithmIdentifier<'_>,
-) -> CryptographyResult<&'p pyo3::PyAny> {
+) -> CryptographyResult<pyo3::Bound<'p, pyo3::PyAny>> {
     match &signature_algorithm.params {
         common::AlgorithmParameters::RsaPss(opt_pss) => {
             let pss = opt_pss.as_ref().ok_or_else(|| {
@@ -586,16 +489,8 @@ pub(crate) fn identify_signature_algorithm_parameters<'p>(
             }
             let py_mask_gen_hash_alg =
                 hash_oid_py_hash(py, pss.mask_gen_algorithm.params.oid().clone())?;
-            let padding = py.import(pyo3::intern!(
-                py,
-                "cryptography.hazmat.primitives.asymmetric.padding"
-            ))?;
-            let py_mgf = padding
-                .getattr(pyo3::intern!(py, "MGF1"))?
-                .call1((py_mask_gen_hash_alg,))?;
-            Ok(padding
-                .getattr(pyo3::intern!(py, "PSS"))?
-                .call1((py_mgf, pss.salt_length))?)
+            let py_mgf = types::MGF1.get(py)?.call1((py_mask_gen_hash_alg,))?;
+            Ok(types::PSS.get(py)?.call1((py_mgf, pss.salt_length))?)
         }
         common::AlgorithmParameters::RsaWithSha1(_)
         | common::AlgorithmParameters::RsaWithSha1Alt(_)
@@ -607,14 +502,7 @@ pub(crate) fn identify_signature_algorithm_parameters<'p>(
         | common::AlgorithmParameters::RsaWithSha3_256(_)
         | common::AlgorithmParameters::RsaWithSha3_384(_)
         | common::AlgorithmParameters::RsaWithSha3_512(_) => {
-            let pkcs = py
-                .import(pyo3::intern!(
-                    py,
-                    "cryptography.hazmat.primitives.asymmetric.padding"
-                ))?
-                .getattr(pyo3::intern!(py, "PKCS1v15"))?
-                .call0()?;
-            Ok(pkcs)
+            Ok(types::PKCS1V15.get(py)?.call0()?)
         }
         common::AlgorithmParameters::EcDsaWithSha224(_)
         | common::AlgorithmParameters::EcDsaWithSha256(_)
@@ -627,25 +515,20 @@ pub(crate) fn identify_signature_algorithm_parameters<'p>(
             let signature_hash_algorithm =
                 identify_signature_hash_algorithm(py, signature_algorithm)?;
 
-            Ok(py
-                .import(pyo3::intern!(
-                    py,
-                    "cryptography.hazmat.primitives.asymmetric.ec"
-                ))?
-                .getattr(pyo3::intern!(py, "ECDSA"))?
-                .call1((signature_hash_algorithm,))?)
+            Ok(types::ECDSA.get(py)?.call1((signature_hash_algorithm,))?)
         }
-        _ => Ok(py.None().into_ref(py)),
+        _ => Ok(py.None().into_bound(py)),
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use cryptography_x509::{common, oid};
+
     use super::{
         identify_alg_params_for_hash_type, identify_key_type_for_algorithm_params, HashType,
         KeyType,
     };
-    use cryptography_x509::{common, oid};
 
     #[test]
     fn test_identify_key_type_for_algorithm_params() {
@@ -704,10 +587,22 @@ mod tests {
             (&common::AlgorithmParameters::EcDsaWithSha3_512, KeyType::Ec),
             (&common::AlgorithmParameters::Ed25519, KeyType::Ed25519),
             (&common::AlgorithmParameters::Ed448, KeyType::Ed448),
-            (&common::AlgorithmParameters::DsaWithSha224, KeyType::Dsa),
-            (&common::AlgorithmParameters::DsaWithSha256, KeyType::Dsa),
-            (&common::AlgorithmParameters::DsaWithSha384, KeyType::Dsa),
-            (&common::AlgorithmParameters::DsaWithSha512, KeyType::Dsa),
+            (
+                &common::AlgorithmParameters::DsaWithSha224(None),
+                KeyType::Dsa,
+            ),
+            (
+                &common::AlgorithmParameters::DsaWithSha256(None),
+                KeyType::Dsa,
+            ),
+            (
+                &common::AlgorithmParameters::DsaWithSha384(None),
+                KeyType::Dsa,
+            ),
+            (
+                &common::AlgorithmParameters::DsaWithSha512(None),
+                KeyType::Dsa,
+            ),
         ] {
             assert_eq!(
                 identify_key_type_for_algorithm_params(params).unwrap(),
