@@ -2,8 +2,8 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-
 import binascii
+import copy
 import itertools
 import os
 import textwrap
@@ -12,9 +12,14 @@ from binascii import hexlify
 
 import pytest
 
-from cryptography import exceptions, x509
+from cryptography import exceptions, utils, x509
+from cryptography.hazmat.bindings._rust import openssl as rust_openssl
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec
+from cryptography.hazmat.primitives.asymmetric.ec import (
+    EllipticCurvePrivateKey,
+    EllipticCurvePublicKey,
+)
 from cryptography.hazmat.primitives.asymmetric.utils import (
     Prehashed,
     encode_dss_signature,
@@ -26,6 +31,7 @@ from ...utils import (
     load_fips_ecdsa_signing_vectors,
     load_kasvs_ecdh_vectors,
     load_nist_vectors,
+    load_rfc6979_vectors,
     load_vectors_from_file,
     raises_unsupported_algorithm,
 )
@@ -41,23 +47,20 @@ _HASH_TYPES: typing.Dict[str, typing.Type[hashes.HashAlgorithm]] = {
 }
 
 
-def _skip_ecdsa_vector(backend, curve_type, hash_type):
+def _skip_ecdsa_vector(backend, curve: ec.EllipticCurve, hash_type):
     if not backend.elliptic_curve_signature_algorithm_supported(
-        ec.ECDSA(hash_type()), curve_type()
+        ec.ECDSA(hash_type()), curve
     ):
         pytest.skip(
-            "ECDSA not supported with this hash {} and curve {}.".format(
-                hash_type().name, curve_type().name
-            )
+            f"ECDSA not supported with this hash {hash_type().name} and "
+            f"curve {curve.name}."
         )
 
 
-def _skip_curve_unsupported(backend, curve):
+def _skip_curve_unsupported(backend, curve: ec.EllipticCurve):
     if not backend.elliptic_curve_supported(curve):
         pytest.skip(
-            "Curve {} is not supported by this backend {}".format(
-                curve.name, backend
-            )
+            f"Curve {curve.name} is not supported by this backend {backend}"
         )
 
 
@@ -66,9 +69,7 @@ def _skip_exchange_algorithm_unsupported(backend, algorithm, curve):
         algorithm, curve
     ):
         pytest.skip(
-            "Exchange with {} curve is not supported by {}".format(
-                curve.name, backend
-            )
+            f"Exchange with {curve.name} curve is not supported by {backend}"
         )
 
 
@@ -99,7 +100,7 @@ def test_skip_exchange_algorithm_unsupported(backend):
 
 def test_skip_ecdsa_vector(backend):
     with pytest.raises(pytest.skip.Exception):
-        _skip_ecdsa_vector(backend, DummyCurve, hashes.SHA256)
+        _skip_ecdsa_vector(backend, DummyCurve(), hashes.SHA256)
 
 
 def test_derive_private_key_success(backend):
@@ -134,7 +135,12 @@ def test_derive_point_at_infinity(backend):
     _skip_curve_unsupported(backend, curve)
     # order of the curve
     q = 0xFFFFFFFF00000000FFFFFFFFFFFFFFFFBCE6FAADA7179E84F3B9CAC2FC632551
-    with pytest.raises(ValueError, match="Unable to derive"):
+    # BoringSSL rejects infinity points before it ever gets to us, so it
+    # uses a more generic error message.
+    match = (
+        "infinity" if not rust_openssl.CRYPTOGRAPHY_IS_BORINGSSL else "Invalid"
+    )
+    with pytest.raises(ValueError, match=match):
         ec.derive_private_key(q, ec.SECP256R1())
 
 
@@ -172,7 +178,9 @@ def test_invalid_private_numbers_public_numbers():
 
 def test_ec_public_numbers_repr():
     pn = ec.EllipticCurvePublicNumbers(2, 3, ec.SECP256R1())
-    assert repr(pn) == "<EllipticCurvePublicNumbers(curve=secp256r1, x=2, y=3>"
+    assert (
+        repr(pn) == "<EllipticCurvePublicNumbers(curve=secp256r1, x=2, y=3)>"
+    )
 
 
 def test_ec_public_numbers_hash():
@@ -207,6 +215,16 @@ def test_ec_key_key_size(backend):
     assert key.public_key().key_size == 256
 
 
+def test_deprecated_generate_private_key_with_curve_class(backend):
+    # This test verifies that if you pass a curve _class_ instead of instance,
+    # you get a warning and then `key.curve` is still an instance.
+    _skip_curve_unsupported(backend, ec.SECP256R1())
+
+    with pytest.warns(utils.DeprecatedIn42):
+        key = ec.generate_private_key(ec.SECP256R1)  # type: ignore[arg-type]
+    assert isinstance(key.curve, ec.SECP256R1)
+
+
 class TestECWithNumbers:
     def test_with_numbers(self, backend, subtests):
         vectors = itertools.product(
@@ -220,16 +238,14 @@ class TestECWithNumbers:
         )
         for vector, hash_type in vectors:
             with subtests.test():
-                curve_type: typing.Type[ec.EllipticCurve] = ec._CURVE_TYPES[
-                    vector["curve"]
-                ]
+                curve = ec._CURVE_TYPES[vector["curve"]]
 
-                _skip_ecdsa_vector(backend, curve_type, hash_type)
+                _skip_ecdsa_vector(backend, curve, hash_type)
 
                 key = ec.EllipticCurvePrivateNumbers(
                     vector["d"],
                     ec.EllipticCurvePublicNumbers(
-                        vector["x"], vector["y"], curve_type()
+                        vector["x"], vector["y"], curve
                     ),
                 ).private_key(backend)
                 assert key
@@ -238,7 +254,7 @@ class TestECWithNumbers:
                 assert priv_num.private_value == vector["d"]
                 assert priv_num.public_numbers.x == vector["x"]
                 assert priv_num.public_numbers.y == vector["y"]
-                assert curve_type().name == priv_num.public_numbers.curve.name
+                assert curve.name == priv_num.public_numbers.curve.name
 
 
 class TestECDSAVectors:
@@ -254,14 +270,14 @@ class TestECDSAVectors:
         )
         for vector, hash_type in vectors:
             with subtests.test():
-                curve_type = ec._CURVE_TYPES[vector["curve"]]
+                curve = ec._CURVE_TYPES[vector["curve"]]
 
-                _skip_ecdsa_vector(backend, curve_type, hash_type)
+                _skip_ecdsa_vector(backend, curve, hash_type)
 
                 key = ec.EllipticCurvePrivateNumbers(
                     vector["d"],
                     ec.EllipticCurvePublicNumbers(
-                        vector["x"], vector["y"], curve_type()
+                        vector["x"], vector["y"], curve
                     ),
                 ).private_key(backend)
                 assert key
@@ -279,16 +295,16 @@ class TestECDSAVectors:
 
     @pytest.mark.parametrize("curve", ec._CURVE_TYPES.values())
     def test_generate_vector_curves(self, backend, curve):
-        _skip_curve_unsupported(backend, curve())
+        _skip_curve_unsupported(backend, curve)
 
-        key = ec.generate_private_key(curve(), backend)
+        key = ec.generate_private_key(curve, backend)
         assert key
-        assert isinstance(key.curve, curve)
+        assert type(key.curve) is type(curve)
         assert key.curve.key_size
 
         pkey = key.public_key()
         assert pkey
-        assert isinstance(pkey.curve, curve)
+        assert type(pkey.curve) is type(curve)
         assert key.curve.key_size == pkey.curve.key_size
 
     def test_generate_unknown_curve(self, backend):
@@ -412,7 +428,7 @@ class TestECDSAVectors:
         # uses a more generic error message.
         match = (
             r"infinity|invalid form"
-            if not backend._lib.CRYPTOGRAPHY_IS_BORINGSSL
+            if not rust_openssl.CRYPTOGRAPHY_IS_BORINGSSL
             else None
         )
         with pytest.raises(ValueError, match=match):
@@ -456,14 +472,12 @@ class TestECDSAVectors:
         for vector in vectors:
             with subtests.test():
                 hash_type = _HASH_TYPES[vector["digest_algorithm"]]
-                curve_type: typing.Type[ec.EllipticCurve] = ec._CURVE_TYPES[
-                    vector["curve"]
-                ]
+                curve = ec._CURVE_TYPES[vector["curve"]]
 
-                _skip_ecdsa_vector(backend, curve_type, hash_type)
+                _skip_ecdsa_vector(backend, curve, hash_type)
 
                 key = ec.EllipticCurvePublicNumbers(
-                    vector["x"], vector["y"], curve_type()
+                    vector["x"], vector["y"], curve
                 ).public_key(backend)
 
                 signature = encode_dss_signature(vector["r"], vector["s"])
@@ -478,12 +492,12 @@ class TestECDSAVectors:
         for vector in vectors:
             with subtests.test():
                 hash_type = _HASH_TYPES[vector["digest_algorithm"]]
-                curve_type = ec._CURVE_TYPES[vector["curve"]]
+                curve = ec._CURVE_TYPES[vector["curve"]]
 
-                _skip_ecdsa_vector(backend, curve_type, hash_type)
+                _skip_ecdsa_vector(backend, curve, hash_type)
 
                 key = ec.EllipticCurvePublicNumbers(
-                    vector["x"], vector["y"], curve_type()
+                    vector["x"], vector["y"], curve
                 ).public_key(backend)
 
                 signature = encode_dss_signature(vector["r"], vector["s"])
@@ -498,31 +512,122 @@ class TestECDSAVectors:
                         signature, vector["message"], ec.ECDSA(hash_type())
                     )
 
+    def test_unsupported_deterministic_nonce(self, backend):
+        if backend.ecdsa_deterministic_supported():
+            pytest.skip(
+                f"ECDSA deterministic signing is supported by this"
+                f" backend {backend}"
+            )
+        with pytest.raises(exceptions.UnsupportedAlgorithm):
+            ec.ECDSA(hashes.SHA256(), deterministic_signing=True)
+
+    def test_deterministic_nonce(self, backend, subtests):
+        if not backend.ecdsa_deterministic_supported():
+            pytest.skip(
+                f"ECDSA deterministic signing is not supported by this"
+                f" backend {backend}"
+            )
+
+        supported_hash_algorithms = {
+            "SHA1": hashes.SHA1(),
+            "SHA224": hashes.SHA224(),
+            "SHA256": hashes.SHA256(),
+            "SHA384": hashes.SHA384(),
+            "SHA512": hashes.SHA512(),
+        }
+        curves = {
+            "B-163": ec.SECT163R2(),
+            "B-233": ec.SECT233R1(),
+            "B-283": ec.SECT283R1(),
+            "B-409": ec.SECT409R1(),
+            "B-571": ec.SECT571R1(),
+            "K-163": ec.SECT163K1(),
+            "K-233": ec.SECT233K1(),
+            "K-283": ec.SECT283K1(),
+            "K-409": ec.SECT409K1(),
+            "K-571": ec.SECT571K1(),
+            "P-192": ec.SECP192R1(),
+            "P-224": ec.SECP224R1(),
+            "P-256": ec.SECP256R1(),
+            "P-384": ec.SECP384R1(),
+            "P-521": ec.SECP521R1(),
+        }
+        vectors = load_vectors_from_file(
+            os.path.join(
+                "asymmetric", "ECDSA", "RFC6979", "evppkey_ecdsa_rfc6979.txt"
+            ),
+            load_rfc6979_vectors,
+        )
+
+        for vector in vectors:
+            with subtests.test():
+                input = bytes(vector["input"], "utf-8")
+                output = bytes.fromhex(vector["output"])
+                key = bytes("\n".join(vector["key"]), "utf-8")
+                curve = curves[vector["key_name"].split("_")[0]]
+                _skip_curve_unsupported(backend, curve)
+
+                if "digest_sign" in vector:
+                    algorithm = vector["digest_sign"]
+                    hash_algorithm = supported_hash_algorithms[algorithm]
+                    algorithm = ec.ECDSA(
+                        hash_algorithm,
+                        deterministic_signing=vector["deterministic_nonce"],
+                    )
+                    private_key = serialization.load_pem_private_key(
+                        key, password=None
+                    )
+                    assert isinstance(private_key, EllipticCurvePrivateKey)
+                    signature = private_key.sign(input, algorithm)
+                    assert signature == output
+                else:
+                    assert "digest_verify" in vector
+                    algorithm = vector["digest_verify"]
+                    assert algorithm in supported_hash_algorithms
+                    hash_algorithm = supported_hash_algorithms[algorithm]
+                    algorithm = ec.ECDSA(hash_algorithm)
+                    public_key = serialization.load_pem_public_key(key)
+                    assert isinstance(public_key, EllipticCurvePublicKey)
+                    if vector["verify_error"]:
+                        with pytest.raises(exceptions.InvalidSignature):
+                            public_key.verify(output, input, algorithm)
+                    else:
+                        public_key.verify(output, input, algorithm)
+
     def test_sign(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         message = b"one little message"
-        algorithm = ec.ECDSA(hashes.SHA1())
+        algorithm = ec.ECDSA(hashes.SHA256())
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
         signature = private_key.sign(message, algorithm)
         public_key = private_key.public_key()
         public_key.verify(signature, message, algorithm)
 
+    def test_sign_verify_buffers(self, backend):
+        _skip_curve_unsupported(backend, ec.SECP256R1())
+        message = bytearray(b"one little message")
+        algorithm = ec.ECDSA(hashes.SHA256())
+        private_key = ec.generate_private_key(ec.SECP256R1(), backend)
+        signature = private_key.sign(message, algorithm)
+        public_key = private_key.public_key()
+        public_key.verify(bytearray(signature), message, algorithm)
+
     def test_sign_prehashed(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         message = b"one little message"
-        h = hashes.Hash(hashes.SHA1(), backend)
+        h = hashes.Hash(hashes.SHA256(), backend)
         h.update(message)
         data = h.finalize()
-        algorithm = ec.ECDSA(Prehashed(hashes.SHA1()))
+        algorithm = ec.ECDSA(Prehashed(hashes.SHA256()))
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
         signature = private_key.sign(data, algorithm)
         public_key = private_key.public_key()
-        public_key.verify(signature, message, ec.ECDSA(hashes.SHA1()))
+        public_key.verify(signature, message, ec.ECDSA(hashes.SHA256()))
 
     def test_sign_prehashed_digest_mismatch(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         message = b"one little message"
-        h = hashes.Hash(hashes.SHA1(), backend)
+        h = hashes.Hash(hashes.SHA224(), backend)
         h.update(message)
         data = h.finalize()
         algorithm = ec.ECDSA(Prehashed(hashes.SHA256()))
@@ -533,7 +638,7 @@ class TestECDSAVectors:
     def test_verify(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         message = b"one little message"
-        algorithm = ec.ECDSA(hashes.SHA1())
+        algorithm = ec.ECDSA(hashes.SHA256())
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
         signature = private_key.sign(message, algorithm)
         public_key = private_key.public_key()
@@ -542,20 +647,22 @@ class TestECDSAVectors:
     def test_verify_prehashed(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         message = b"one little message"
-        algorithm = ec.ECDSA(hashes.SHA1())
+        algorithm = ec.ECDSA(hashes.SHA256())
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
         signature = private_key.sign(message, algorithm)
-        h = hashes.Hash(hashes.SHA1(), backend)
+        h = hashes.Hash(hashes.SHA256(), backend)
         h.update(message)
         data = h.finalize()
         public_key = private_key.public_key()
-        public_key.verify(signature, data, ec.ECDSA(Prehashed(hashes.SHA1())))
+        public_key.verify(
+            signature, data, ec.ECDSA(Prehashed(hashes.SHA256()))
+        )
 
     def test_verify_prehashed_digest_mismatch(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         message = b"one little message"
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
-        h = hashes.Hash(hashes.SHA1(), backend)
+        h = hashes.Hash(hashes.SHA224(), backend)
         h.update(message)
         data = h.finalize()
         public_key = private_key.public_key()
@@ -613,6 +720,19 @@ class TestECEquality:
         assert key1 == key2
         assert key1 != key3
         assert key1 != object()
+        with pytest.raises(TypeError):
+            key1 < key2  # type: ignore[operator]
+
+    def test_public_key_copy(self, backend):
+        _skip_curve_unsupported(backend, ec.SECP256R1())
+        key_bytes = load_vectors_from_file(
+            os.path.join("asymmetric", "PKCS8", "ec_private_key.pem"),
+            lambda pemfile: pemfile.read().encode(),
+        )
+        key1 = serialization.load_pem_private_key(key_bytes, None).public_key()
+        key2 = copy.copy(key1)
+
+        assert key1 == key2
 
 
 class TestECSerialization:
@@ -653,6 +773,24 @@ class TestECSerialization:
         priv_num = key.private_numbers()
         assert loaded_priv_num == priv_num
 
+    @pytest.mark.supported(
+        only_if=lambda backend: backend._fips_enabled,
+        skip_message="Requires FIPS",
+    )
+    def test_traditional_serialization_fips(self, backend):
+        key_bytes = load_vectors_from_file(
+            os.path.join("asymmetric", "PKCS8", "ec_private_key.pem"),
+            lambda pemfile: pemfile.read().encode(),
+        )
+        key = serialization.load_pem_private_key(key_bytes, None, backend)
+        assert isinstance(key, ec.EllipticCurvePrivateKey)
+        with pytest.raises(ValueError):
+            key.private_bytes(
+                serialization.Encoding.PEM,
+                serialization.PrivateFormat.TraditionalOpenSSL,
+                serialization.BestAvailableEncryption(b"password"),
+            )
+
     @pytest.mark.parametrize(
         ("encoding", "fmt"),
         [
@@ -794,6 +932,13 @@ class TestECSerialization:
                 serialization.BestAvailableEncryption(b"password"),
             )
 
+        with pytest.raises(ValueError):
+            key.private_bytes(
+                serialization.Encoding.SMIME,
+                serialization.PrivateFormat.TraditionalOpenSSL,
+                serialization.NoEncryption(),
+            )
+
     def test_private_bytes_invalid_encoding(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         key = load_vectors_from_file(
@@ -870,6 +1015,62 @@ class TestECSerialization:
         parsed_public = serialization.load_pem_public_key(pem, backend)
         assert parsed_public
 
+    def test_load_private_key_explicit_parameters(self):
+        with pytest.raises(ValueError, match="explicit parameters"):
+            load_vectors_from_file(
+                os.path.join(
+                    "asymmetric", "EC", "explicit_parameters_private_key.pem"
+                ),
+                lambda pemfile: serialization.load_pem_private_key(
+                    pemfile.read(), password=None
+                ),
+                mode="rb",
+            )
+
+        with pytest.raises(ValueError, match="explicit parameters"):
+            load_vectors_from_file(
+                os.path.join(
+                    "asymmetric",
+                    "EC",
+                    "explicit_parameters_wap_wsg_idm_ecid_wtls11_private_key.pem",
+                ),
+                lambda pemfile: serialization.load_pem_private_key(
+                    pemfile.read(), password=None
+                ),
+                mode="rb",
+            )
+
+    def test_load_private_key_unsupported_curve(self):
+        with pytest.raises((ValueError, exceptions.UnsupportedAlgorithm)):
+            load_vectors_from_file(
+                os.path.join("asymmetric", "EC", "secp128r1_private_key.pem"),
+                lambda pemfile: serialization.load_pem_private_key(
+                    pemfile.read(), password=None
+                ),
+                mode="rb",
+            )
+
+    @pytest.mark.parametrize(
+        ("key_file", "curve"),
+        [
+            ("sect163k1-spki.pem", ec.SECT163K1),
+            ("sect163r2-spki.pem", ec.SECT163R2),
+            ("sect233k1-spki.pem", ec.SECT233K1),
+            ("sect233r1-spki.pem", ec.SECT233R1),
+        ],
+    )
+    def test_load_public_keys(self, key_file, curve, backend):
+        _skip_curve_unsupported(backend, curve())
+        key = load_vectors_from_file(
+            os.path.join("asymmetric", "EC", key_file),
+            lambda pemfile: serialization.load_pem_public_key(
+                pemfile.read(),
+            ),
+            mode="rb",
+        )
+        assert isinstance(key, ec.EllipticCurvePublicKey)
+        assert isinstance(key.curve, curve)
+
 
 class TestEllipticCurvePEMPublicKeySerialization:
     @pytest.mark.parametrize(
@@ -1077,7 +1278,8 @@ class TestEllipticCurvePEMPublicKeySerialization:
     def test_from_encoded_point_not_a_curve(self):
         with pytest.raises(TypeError):
             ec.EllipticCurvePublicKey.from_encoded_point(
-                "notacurve", b"\x04data"  # type: ignore[arg-type]
+                "notacurve",  # type: ignore[arg-type]
+                b"\x04data",
             )
 
     def test_from_encoded_point_unsupported_encoding(self):
@@ -1140,7 +1342,7 @@ class TestECDH:
         for vector in vectors:
             with subtests.test():
                 _skip_exchange_algorithm_unsupported(
-                    backend, ec.ECDH(), ec._CURVE_TYPES[vector["curve"]]()
+                    backend, ec.ECDH(), ec._CURVE_TYPES[vector["curve"]]
                 )
 
                 key_numbers = vector["IUT"]
@@ -1149,7 +1351,7 @@ class TestECDH:
                     ec.EllipticCurvePublicNumbers(
                         key_numbers["x"],
                         key_numbers["y"],
-                        ec._CURVE_TYPES[vector["curve"]](),
+                        ec._CURVE_TYPES[vector["curve"]],
                     ),
                 )
                 # Errno 5-7 indicates a bad public or private key, this
@@ -1165,7 +1367,7 @@ class TestECDH:
                 public_numbers = ec.EllipticCurvePublicNumbers(
                     peer_numbers["x"],
                     peer_numbers["y"],
-                    ec._CURVE_TYPES[vector["curve"]](),
+                    ec._CURVE_TYPES[vector["curve"]],
                 )
                 # Errno 1 and 2 indicates a bad public key, this doesn't test
                 # the ECDH code at all
@@ -1195,7 +1397,7 @@ class TestECDH:
         ),
     )
     def test_brainpool_kex(self, backend, vector):
-        curve = ec._CURVE_TYPES[vector["curve"].decode("ascii")]()
+        curve = ec._CURVE_TYPES[vector["curve"].decode("ascii")]
         _skip_exchange_algorithm_unsupported(backend, ec.ECDH(), curve)
         key = ec.EllipticCurvePrivateNumbers(
             int(vector["da"], 16),
