@@ -2,17 +2,14 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
 import pytest
 
-import six
-
 from cryptography.exceptions import AlreadyFinalized
 from cryptography.hazmat.primitives import padding
 
 
-class TestPKCS7(object):
+class TestPKCS7:
     @pytest.mark.parametrize("size", [127, 4096, -2])
     def test_invalid_block_size(self, size):
         with pytest.raises(ValueError):
@@ -38,10 +35,10 @@ class TestPKCS7(object):
     def test_non_bytes(self):
         padder = padding.PKCS7(128).padder()
         with pytest.raises(TypeError):
-            padder.update(u"abc")
+            padder.update("abc")  # type: ignore[arg-type]
         unpadder = padding.PKCS7(128).unpadder()
         with pytest.raises(TypeError):
-            unpadder.update(u"abc")
+            unpadder.update("abc")  # type: ignore[arg-type]
 
     def test_zany_py2_bytes_subclass(self):
         class mybytes(bytes):  # noqa: N801
@@ -50,9 +47,9 @@ class TestPKCS7(object):
 
         str(mybytes())
         padder = padding.PKCS7(128).padder()
-        padder.update(mybytes(b"abc"))
+        data = padder.update(mybytes(b"abc")) + padder.finalize()
         unpadder = padding.PKCS7(128).unpadder()
-        unpadder.update(mybytes(padder.finalize()))
+        unpadder.update(mybytes(data))
         assert unpadder.finalize() == b"abc"
 
     @pytest.mark.parametrize(
@@ -65,7 +62,7 @@ class TestPKCS7(object):
                 b"111111111111111122222222222222\x02\x02",
             ),
             (128, b"1" * 16, b"1" * 16 + b"\x10" * 16),
-            (128, b"1" * 17, b"1" * 17 + b"\x0F" * 15),
+            (128, b"1" * 17, b"1" * 17 + b"\x0f" * 15),
         ],
     )
     def test_pad(self, size, unpadded, padded):
@@ -112,7 +109,7 @@ class TestPKCS7(object):
         padded_data = padder.update(b"")
         padded_data += padder.finalize()
 
-        for i in six.iterbytes(padded_data):
+        for i in padded_data:
             assert i == 255
 
         unpadder = padding.PKCS7(2040).unpadder()
@@ -134,7 +131,7 @@ class TestPKCS7(object):
         assert final == unpadded + unpadded
 
 
-class TestANSIX923(object):
+class TestANSIX923:
     @pytest.mark.parametrize("size", [127, 4096, -2])
     def test_invalid_block_size(self, size):
         with pytest.raises(ValueError):
@@ -161,10 +158,10 @@ class TestANSIX923(object):
     def test_non_bytes(self):
         padder = padding.ANSIX923(128).padder()
         with pytest.raises(TypeError):
-            padder.update(u"abc")
+            padder.update("abc")  # type: ignore[arg-type]
         unpadder = padding.ANSIX923(128).unpadder()
         with pytest.raises(TypeError):
-            unpadder.update(u"abc")
+            unpadder.update("abc")  # type: ignore[arg-type]
 
     def test_zany_py2_bytes_subclass(self):
         class mybytes(bytes):  # noqa: N801
@@ -188,7 +185,7 @@ class TestANSIX923(object):
                 b"111111111111111122222222222222\x00\x02",
             ),
             (128, b"1" * 16, b"1" * 16 + b"\x00" * 15 + b"\x10"),
-            (128, b"1" * 17, b"1" * 17 + b"\x00" * 14 + b"\x0F"),
+            (128, b"1" * 17, b"1" * 17 + b"\x00" * 14 + b"\x0f"),
         ],
     )
     def test_pad(self, size, unpadded, padded):
