@@ -11,8 +11,9 @@ import pytest
 
 from cryptography import x509
 from cryptography.exceptions import _Reasons
+from cryptography.hazmat.bindings._rust import test_support
 from cryptography.hazmat.primitives import hashes, serialization
-from cryptography.hazmat.primitives.asymmetric import ed25519, rsa
+from cryptography.hazmat.primitives.asymmetric import ed25519, padding, rsa
 from cryptography.hazmat.primitives.serialization import pkcs7
 
 from ...utils import load_vectors_from_file, raises_unsupported_algorithm
@@ -89,67 +90,11 @@ class TestPKCS7Loading:
                 mode="rb",
             )
 
-    def test_load_pkcs7_empty_certificates(self, backend):
-        der = b"\x30\x0B\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x07\x02"
+    def test_load_pkcs7_empty_certificates(self):
+        der = b"\x30\x0b\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01\x07\x02"
 
-        certificates = pkcs7.load_der_pkcs7_certificates(der)
-        assert certificates == []
-
-
-# We have no public verification API and won't be adding one until we get
-# some requirements from users so this function exists to give us basic
-# verification for the signing tests.
-def _pkcs7_verify(encoding, sig, msg, certs, options, backend):
-    sig_bio = backend._bytes_to_bio(sig)
-    if encoding is serialization.Encoding.DER:
-        p7 = backend._lib.d2i_PKCS7_bio(sig_bio.bio, backend._ffi.NULL)
-    elif encoding is serialization.Encoding.PEM:
-        p7 = backend._lib.PEM_read_bio_PKCS7(
-            sig_bio.bio,
-            backend._ffi.NULL,
-            backend._ffi.NULL,
-            backend._ffi.NULL,
-        )
-    else:
-        p7 = backend._lib.SMIME_read_PKCS7(sig_bio.bio, backend._ffi.NULL)
-    backend.openssl_assert(p7 != backend._ffi.NULL)
-    p7 = backend._ffi.gc(p7, backend._lib.PKCS7_free)
-    flags = 0
-    for option in options:
-        if option is pkcs7.PKCS7Options.Text:
-            flags |= backend._lib.PKCS7_TEXT
-    store = backend._lib.X509_STORE_new()
-    backend.openssl_assert(store != backend._ffi.NULL)
-    store = backend._ffi.gc(store, backend._lib.X509_STORE_free)
-    # This list is to keep the x509 values alive until end of function
-    ossl_certs = []
-    for cert in certs:
-        ossl_cert = backend._cert2ossl(cert)
-        ossl_certs.append(ossl_cert)
-        res = backend._lib.X509_STORE_add_cert(store, ossl_cert)
-        backend.openssl_assert(res == 1)
-    if msg is None:
-        res = backend._lib.PKCS7_verify(
-            p7,
-            backend._ffi.NULL,
-            store,
-            backend._ffi.NULL,
-            backend._ffi.NULL,
-            flags,
-        )
-    else:
-        msg_bio = backend._bytes_to_bio(msg)
-        # libressl 3.7.0 has a bug when NULL is passed as an `out_bio`. Work
-        # around it for now.
-        out_bio = backend._create_mem_bio_gc()
-        res = backend._lib.PKCS7_verify(
-            p7, backend._ffi.NULL, store, msg_bio.bio, out_bio, flags
-        )
-    backend.openssl_assert(res == 1)
-    # OpenSSL 3.0 leaves a random bio error on the stack:
-    # https://github.com/openssl/openssl/issues/16681
-    if backend._lib.CRYPTOGRAPHY_OPENSSL_300_OR_GREATER:
-        backend._consume_errors()
+        with pytest.raises(ValueError):
+            pkcs7.load_der_pkcs7_certificates(der)
 
 
 def _load_cert_key():
@@ -172,7 +117,7 @@ def _load_cert_key():
     only_if=lambda backend: backend.pkcs7_supported(),
     skip_message="Requires OpenSSL with PKCS7 support",
 )
-class TestPKCS7Builder:
+class TestPKCS7SignatureBuilder:
     def test_invalid_data(self, backend):
         builder = pkcs7.PKCS7SignatureBuilder()
         with pytest.raises(TypeError):
@@ -200,14 +145,18 @@ class TestPKCS7Builder:
         cert, key = _load_cert_key()
         with pytest.raises(TypeError):
             pkcs7.PKCS7SignatureBuilder().add_signer(
-                cert, key, hashes.SHA512_256()  # type: ignore[arg-type]
+                cert,
+                key,
+                hashes.SHA512_256(),  # type: ignore[arg-type]
             )
 
     def test_not_a_cert(self, backend):
-        cert, key = _load_cert_key()
+        _, key = _load_cert_key()
         with pytest.raises(TypeError):
             pkcs7.PKCS7SignatureBuilder().add_signer(
-                b"notacert", key, hashes.SHA256()  # type: ignore[arg-type]
+                b"notacert",  # type: ignore[arg-type]
+                key,
+                hashes.SHA256(),
             )
 
     @pytest.mark.supported(
@@ -219,7 +168,9 @@ class TestPKCS7Builder:
         key = ed25519.Ed25519PrivateKey.generate()
         with pytest.raises(TypeError):
             pkcs7.PKCS7SignatureBuilder().add_signer(
-                cert, key, hashes.SHA256()  # type: ignore[arg-type]
+                cert,
+                key,  # type: ignore[arg-type]
+                hashes.SHA256(),
             )
 
     def test_sign_invalid_options(self, backend):
@@ -309,23 +260,25 @@ class TestPKCS7Builder:
         # Parse the message to get the signed data, which is the
         # first payload in the message
         message = email.parser.BytesParser().parsebytes(sig)
-        signed_data = message.get_payload()[0].get_payload().encode()
-        _pkcs7_verify(
+        payload = message.get_payload()
+        assert isinstance(payload, list)
+        assert isinstance(payload[0], email.message.Message)
+        signed_data = payload[0].get_payload()
+        assert isinstance(signed_data, str)
+        test_support.pkcs7_verify(
             serialization.Encoding.SMIME,
             sig,
-            signed_data,
+            signed_data.encode(),
             [cert],
             options,
-            backend,
         )
         assert data not in sig_binary
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig_binary,
             data,
             [cert],
             options,
-            backend,
         )
 
     def test_sign_byteslike(self, backend):
@@ -340,13 +293,12 @@ class TestPKCS7Builder:
 
         sig = builder.sign(serialization.Encoding.SMIME, options)
         assert bytes(data) in sig
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.SMIME,
             sig,
             data,
             [cert],
             options,
-            backend,
         )
 
         data = bytearray(b"")
@@ -357,13 +309,12 @@ class TestPKCS7Builder:
         )
 
         sig = builder.sign(serialization.Encoding.SMIME, options)
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.SMIME,
             sig,
             data,
             [cert],
             options,
-            backend,
         )
 
     def test_sign_pem(self, backend):
@@ -377,13 +328,12 @@ class TestPKCS7Builder:
         )
 
         sig = builder.sign(serialization.Encoding.PEM, options)
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.PEM,
             sig,
             None,
             [cert],
             options,
-            backend,
         )
 
     @pytest.mark.parametrize(
@@ -407,8 +357,8 @@ class TestPKCS7Builder:
         options: typing.List[pkcs7.PKCS7Options] = []
         sig = builder.sign(serialization.Encoding.DER, options)
         assert expected_value in sig
-        _pkcs7_verify(
-            serialization.Encoding.DER, sig, None, [cert], options, backend
+        test_support.pkcs7_verify(
+            serialization.Encoding.DER, sig, None, [cert], options
         )
 
     @pytest.mark.parametrize(
@@ -449,13 +399,12 @@ class TestPKCS7Builder:
         # When not passing detached signature the signed data is embedded into
         # the PKCS7 structure itself
         assert data in sig_binary
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig_binary,
             None,
             [cert],
             options,
-            backend,
         )
 
     def test_sign_binary(self, backend):
@@ -475,22 +424,20 @@ class TestPKCS7Builder:
         # so data should not be present in sig_no_binary, but should be present
         # in sig_binary
         assert data not in sig_no_binary
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig_no_binary,
             None,
             [cert],
             options,
-            backend,
         )
         assert data in sig_binary
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig_binary,
             None,
             [cert],
             options,
-            backend,
         )
 
     def test_sign_smime_canonicalization(self, backend):
@@ -508,13 +455,12 @@ class TestPKCS7Builder:
         # so data should not be present in the sig
         assert data not in sig_binary
         assert b"hello\r\nworld" in sig_binary
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig_binary,
             None,
             [cert],
             options,
-            backend,
         )
 
     def test_sign_text(self, backend):
@@ -539,16 +485,61 @@ class TestPKCS7Builder:
         # Parse the message to get the signed data, which is the
         # first payload in the message
         message = email.parser.BytesParser().parsebytes(sig_pem)
-        signed_data = message.get_payload()[0].as_bytes(
+        payload = message.get_payload()
+        assert isinstance(payload, list)
+        assert isinstance(payload[0], email.message.Message)
+        signed_data = payload[0].as_bytes(
             policy=message.policy.clone(linesep="\r\n")
         )
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.SMIME,
             sig_pem,
             signed_data,
             [cert],
             options,
-            backend,
+        )
+
+    def test_smime_capabilities(self, backend):
+        data = b"hello world"
+        cert, key = _load_cert_key()
+        builder = (
+            pkcs7.PKCS7SignatureBuilder()
+            .set_data(data)
+            .add_signer(cert, key, hashes.SHA256())
+        )
+
+        sig_binary = builder.sign(serialization.Encoding.DER, [])
+
+        # 1.2.840.113549.1.9.15 (SMIMECapabilities) as an ASN.1 DER encoded OID
+        assert b"\x06\t*\x86H\x86\xf7\r\x01\t\x0f" in sig_binary
+
+        # 2.16.840.1.101.3.4.1.42 (aes256-CBC-PAD) as an ASN.1 DER encoded OID
+        aes256_cbc_pad_oid = b"\x06\x09\x60\x86\x48\x01\x65\x03\x04\x01\x2a"
+        # 2.16.840.1.101.3.4.1.22 (aes192-CBC-PAD) as an ASN.1 DER encoded OID
+        aes192_cbc_pad_oid = b"\x06\x09\x60\x86\x48\x01\x65\x03\x04\x01\x16"
+        # 2.16.840.1.101.3.4.1.2 (aes128-CBC-PAD) as an ASN.1 DER encoded OID
+        aes128_cbc_pad_oid = b"\x06\x09\x60\x86\x48\x01\x65\x03\x04\x01\x02"
+
+        # Each algorithm in SMIMECapabilities should be inside its own
+        # SEQUENCE.
+        # This is encoded as SEQUENCE_IDENTIFIER + LENGTH + ALGORITHM_OID.
+        # This tests that each algorithm is indeed encoded inside its own
+        # sequence. See RFC 2633, Appendix A for more details.
+        sequence_identifier = b"\x30"
+        for oid in [
+            aes256_cbc_pad_oid,
+            aes192_cbc_pad_oid,
+            aes128_cbc_pad_oid,
+        ]:
+            len_oid = len(oid).to_bytes(length=1, byteorder="big")
+            assert sequence_identifier + len_oid + oid in sig_binary
+
+        test_support.pkcs7_verify(
+            serialization.Encoding.DER,
+            sig_binary,
+            None,
+            [cert],
+            [],
         )
 
     def test_sign_no_capabilities(self, backend):
@@ -571,13 +562,12 @@ class TestPKCS7Builder:
         assert b"\x06\t*\x86H\x86\xf7\r\x01\t\x0f" not in sig_binary
         # 1.2.840.113549.1.9.5 signingTime as an ASN.1 DER encoded OID
         assert b"\x06\t*\x86H\x86\xf7\r\x01\t\x05" in sig_binary
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig_binary,
             None,
             [cert],
             options,
-            backend,
         )
 
     def test_sign_no_attributes(self, backend):
@@ -598,13 +588,12 @@ class TestPKCS7Builder:
         assert b"\x06\t*\x86H\x86\xf7\r\x01\t\x0f" not in sig_binary
         # 1.2.840.113549.1.9.5 signingTime as an ASN.1 DER encoded OID
         assert b"\x06\t*\x86H\x86\xf7\r\x01\t\x05" not in sig_binary
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig_binary,
             None,
             [cert],
             options,
-            backend,
         )
 
     def test_sign_no_certs(self, backend):
@@ -624,6 +613,102 @@ class TestPKCS7Builder:
         sig_no = builder.sign(serialization.Encoding.DER, options)
         assert sig_no.count(cert.public_bytes(serialization.Encoding.DER)) == 0
 
+    @pytest.mark.parametrize(
+        "pad",
+        [
+            padding.PKCS1v15(),
+            None,
+            padding.PSS(
+                mgf=padding.MGF1(hashes.SHA512()),
+                salt_length=padding.PSS.DIGEST_LENGTH,
+            ),
+        ],
+    )
+    def test_rsa_pkcs_padding_options(self, pad, backend):
+        data = b"hello world"
+        rsa_key = load_vectors_from_file(
+            os.path.join("x509", "custom", "ca", "rsa_key.pem"),
+            lambda pemfile: serialization.load_pem_private_key(
+                pemfile.read(), None, unsafe_skip_rsa_key_validation=True
+            ),
+            mode="rb",
+        )
+        assert isinstance(rsa_key, rsa.RSAPrivateKey)
+        rsa_cert = load_vectors_from_file(
+            os.path.join("x509", "custom", "ca", "rsa_ca.pem"),
+            loader=lambda pemfile: x509.load_pem_x509_certificate(
+                pemfile.read()
+            ),
+            mode="rb",
+        )
+        builder = (
+            pkcs7.PKCS7SignatureBuilder()
+            .set_data(data)
+            .add_signer(rsa_cert, rsa_key, hashes.SHA512(), rsa_padding=pad)
+        )
+        options: typing.List[pkcs7.PKCS7Options] = []
+        sig = builder.sign(serialization.Encoding.DER, options)
+        # This should be a pkcs1 sha512 signature
+        if isinstance(pad, padding.PSS):
+            # PKCS7_verify can't verify a PSS sig and we don't bind CMS so
+            # we instead just check that a few things are present in the
+            # output.
+            # There should be four SHA512 OIDs in this structure
+            assert sig.count(b"\x06\t`\x86H\x01e\x03\x04\x02\x03") == 4
+            # There should be one MGF1 OID in this structure
+            assert (
+                sig.count(b"\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01\x01\x08") == 1
+            )
+        else:
+            # This should be a pkcs1 RSA signature, which uses the
+            # `rsaEncryption` OID (1.2.840.113549.1.1.1) no matter which
+            # digest algorithm is used.
+            # See RFC 3370 section 3.2 for more details.
+            # This OID appears twice, once in the certificate itself and
+            # another in the SignerInfo data structure in the
+            # `digest_encryption_algorithm` field.
+            assert (
+                sig.count(b"\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01\x01\x01") == 2
+            )
+            test_support.pkcs7_verify(
+                serialization.Encoding.DER,
+                sig,
+                None,
+                [rsa_cert],
+                options,
+            )
+
+    def test_not_rsa_key_with_padding(self, backend):
+        cert, key = _load_cert_key()
+        with pytest.raises(TypeError):
+            pkcs7.PKCS7SignatureBuilder().add_signer(
+                cert, key, hashes.SHA512(), rsa_padding=padding.PKCS1v15()
+            )
+
+    def test_rsa_invalid_padding(self, backend):
+        rsa_key = load_vectors_from_file(
+            os.path.join("x509", "custom", "ca", "rsa_key.pem"),
+            lambda pemfile: serialization.load_pem_private_key(
+                pemfile.read(), None, unsafe_skip_rsa_key_validation=True
+            ),
+            mode="rb",
+        )
+        assert isinstance(rsa_key, rsa.RSAPrivateKey)
+        rsa_cert = load_vectors_from_file(
+            os.path.join("x509", "custom", "ca", "rsa_ca.pem"),
+            loader=lambda pemfile: x509.load_pem_x509_certificate(
+                pemfile.read()
+            ),
+            mode="rb",
+        )
+        with pytest.raises(TypeError):
+            pkcs7.PKCS7SignatureBuilder().add_signer(
+                rsa_cert,
+                rsa_key,
+                hashes.SHA512(),
+                rsa_padding=object(),  # type: ignore[arg-type]
+            )
+
     def test_multiple_signers(self, backend):
         data = b"hello world"
         cert, key = _load_cert_key()
@@ -652,13 +737,12 @@ class TestPKCS7Builder:
         sig = builder.sign(serialization.Encoding.DER, options)
         # There should be three SHA512 OIDs in this structure
         assert sig.count(b"\x06\t`\x86H\x01e\x03\x04\x02\x03") == 3
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig,
             None,
             [cert, rsa_cert],
             options,
-            backend,
         )
 
     def test_multiple_signers_different_hash_algs(self, backend):
@@ -690,13 +774,12 @@ class TestPKCS7Builder:
         # There should be two SHA384 and two SHA512 OIDs in this structure
         assert sig.count(b"\x06\t`\x86H\x01e\x03\x04\x02\x02") == 2
         assert sig.count(b"\x06\t`\x86H\x01e\x03\x04\x02\x03") == 2
-        _pkcs7_verify(
+        test_support.pkcs7_verify(
             serialization.Encoding.DER,
             sig,
             None,
             [cert, rsa_cert],
             options,
-            backend,
         )
 
     def test_add_additional_cert_not_a_cert(self, backend):
@@ -751,6 +834,242 @@ class TestPKCS7Builder:
         )
 
 
+def _load_rsa_cert_key():
+    key = load_vectors_from_file(
+        os.path.join("x509", "custom", "ca", "rsa_key.pem"),
+        lambda pemfile: serialization.load_pem_private_key(
+            pemfile.read(), None, unsafe_skip_rsa_key_validation=True
+        ),
+        mode="rb",
+    )
+    cert = load_vectors_from_file(
+        os.path.join("x509", "custom", "ca", "rsa_ca.pem"),
+        loader=lambda pemfile: x509.load_pem_x509_certificate(pemfile.read()),
+        mode="rb",
+    )
+    return cert, key
+
+
+@pytest.mark.supported(
+    only_if=lambda backend: backend.pkcs7_supported()
+    and backend.rsa_encryption_supported(padding.PKCS1v15()),
+    skip_message="Requires OpenSSL with PKCS7 support and PKCS1 v1.5 padding "
+    "support",
+)
+class TestPKCS7EnvelopeBuilder:
+    def test_invalid_data(self, backend):
+        builder = pkcs7.PKCS7EnvelopeBuilder()
+        with pytest.raises(TypeError):
+            builder.set_data("not bytes")  # type: ignore[arg-type]
+
+    def test_set_data_twice(self, backend):
+        builder = pkcs7.PKCS7EnvelopeBuilder().set_data(b"test")
+        with pytest.raises(ValueError):
+            builder.set_data(b"test")
+
+    def test_encrypt_no_recipient(self, backend):
+        builder = pkcs7.PKCS7EnvelopeBuilder().set_data(b"test")
+        with pytest.raises(ValueError):
+            builder.encrypt(serialization.Encoding.SMIME, [])
+
+    def test_encrypt_no_data(self, backend):
+        cert, _ = _load_rsa_cert_key()
+        builder = pkcs7.PKCS7EnvelopeBuilder().add_recipient(cert)
+        with pytest.raises(ValueError):
+            builder.encrypt(serialization.Encoding.SMIME, [])
+
+    def test_unsupported_encryption(self, backend):
+        cert_non_rsa, _ = _load_cert_key()
+        with pytest.raises(TypeError):
+            pkcs7.PKCS7EnvelopeBuilder().add_recipient(cert_non_rsa)
+
+    def test_not_a_cert(self, backend):
+        with pytest.raises(TypeError):
+            pkcs7.PKCS7EnvelopeBuilder().add_recipient(
+                b"notacert",  # type: ignore[arg-type]
+            )
+
+    def test_encrypt_invalid_options(self, backend):
+        cert, _ = _load_rsa_cert_key()
+        builder = (
+            pkcs7.PKCS7EnvelopeBuilder().set_data(b"test").add_recipient(cert)
+        )
+        with pytest.raises(ValueError):
+            builder.encrypt(
+                serialization.Encoding.SMIME,
+                [b"invalid"],  # type: ignore[list-item]
+            )
+
+    def test_encrypt_invalid_encoding(self, backend):
+        cert, _ = _load_rsa_cert_key()
+        builder = (
+            pkcs7.PKCS7EnvelopeBuilder().set_data(b"test").add_recipient(cert)
+        )
+        with pytest.raises(ValueError):
+            builder.encrypt(serialization.Encoding.Raw, [])
+
+    @pytest.mark.parametrize(
+        "invalid_options",
+        [
+            [pkcs7.PKCS7Options.NoAttributes],
+            [pkcs7.PKCS7Options.NoCapabilities],
+            [pkcs7.PKCS7Options.NoCerts],
+            [pkcs7.PKCS7Options.DetachedSignature],
+            [pkcs7.PKCS7Options.Binary, pkcs7.PKCS7Options.Text],
+        ],
+    )
+    def test_encrypt_invalid_encryption_options(
+        self, backend, invalid_options
+    ):
+        cert, _ = _load_rsa_cert_key()
+        builder = (
+            pkcs7.PKCS7EnvelopeBuilder().set_data(b"test").add_recipient(cert)
+        )
+        with pytest.raises(ValueError):
+            builder.encrypt(serialization.Encoding.DER, invalid_options)
+
+    @pytest.mark.parametrize(
+        "options",
+        [
+            [pkcs7.PKCS7Options.Text],
+            [pkcs7.PKCS7Options.Binary],
+        ],
+    )
+    def test_smime_encrypt_smime_encoding(self, backend, options):
+        data = b"hello world\n"
+        cert, private_key = _load_rsa_cert_key()
+        builder = (
+            pkcs7.PKCS7EnvelopeBuilder().set_data(data).add_recipient(cert)
+        )
+        enveloped = builder.encrypt(serialization.Encoding.SMIME, options)
+        assert b"MIME-Version: 1.0\n" in enveloped
+        assert b"Content-Transfer-Encoding: base64\n" in enveloped
+        message = email.parser.BytesParser().parsebytes(enveloped)
+        assert message.get_content_disposition() == "attachment"
+        assert message.get_filename() == "smime.p7m"
+        assert message.get_content_type() == "application/pkcs7-mime"
+        assert message.get_param("smime-type") == "enveloped-data"
+        assert message.get_param("name") == "smime.p7m"
+
+        payload = message.get_payload(decode=True)
+        assert isinstance(payload, bytes)
+
+        # We want to know if we've serialized something that has the parameters
+        # we expect, so we match on specific byte strings of OIDs & DER values.
+        # OID 2.16.840.1.101.3.4.1.2 (aes128-CBC)
+        assert b"\x06\x09\x60\x86\x48\x01\x65\x03\x04\x01\x02" in payload
+        # OID 1.2.840.113549.1.1.1 (rsaEncryption (PKCS #1))
+        assert b"\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01\x01\x01" in payload
+        # cryptography CA (the recipient's Common Name)
+        assert (
+            b"\x0c\x0f\x63\x72\x79\x70\x74\x6f\x67\x72\x61\x70\x68\x79"
+            b"\x20\x43\x41"
+        ) in payload
+
+        decrypted_bytes = test_support.pkcs7_decrypt(
+            serialization.Encoding.SMIME,
+            enveloped,
+            private_key,
+            cert,
+            options,
+        )
+        # New lines are canonicalized to '\r\n' when not using Binary
+        expected_data = (
+            data
+            if pkcs7.PKCS7Options.Binary in options
+            else data.replace(b"\n", b"\r\n")
+        )
+        assert decrypted_bytes == expected_data
+
+    @pytest.mark.parametrize(
+        "options",
+        [
+            [pkcs7.PKCS7Options.Text],
+            [pkcs7.PKCS7Options.Binary],
+        ],
+    )
+    def test_smime_encrypt_der_encoding(self, backend, options):
+        data = b"hello world\n"
+        cert, private_key = _load_rsa_cert_key()
+        builder = (
+            pkcs7.PKCS7EnvelopeBuilder().set_data(data).add_recipient(cert)
+        )
+        enveloped = builder.encrypt(serialization.Encoding.DER, options)
+
+        # We want to know if we've serialized something that has the parameters
+        # we expect, so we match on specific byte strings of OIDs & DER values.
+        # OID 2.16.840.1.101.3.4.1.2 (aes128-CBC)
+        assert b"\x06\x09\x60\x86\x48\x01\x65\x03\x04\x01\x02" in enveloped
+        # OID 1.2.840.113549.1.1.1 (rsaEncryption (PKCS #1))
+        assert b"\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01\x01\x01" in enveloped
+        # cryptography CA (the recipient's Common Name)
+        assert (
+            b"\x0c\x0f\x63\x72\x79\x70\x74\x6f\x67\x72\x61\x70\x68\x79"
+            b"\x20\x43\x41"
+        ) in enveloped
+
+        decrypted_bytes = test_support.pkcs7_decrypt(
+            serialization.Encoding.DER,
+            enveloped,
+            private_key,
+            cert,
+            options,
+        )
+        # New lines are canonicalized to '\r\n' when not using Binary
+        expected_data = (
+            data
+            if pkcs7.PKCS7Options.Binary in options
+            else data.replace(b"\n", b"\r\n")
+        )
+        assert decrypted_bytes == expected_data
+
+    @pytest.mark.parametrize(
+        "options",
+        [
+            [pkcs7.PKCS7Options.Text],
+            [pkcs7.PKCS7Options.Binary],
+        ],
+    )
+    def test_smime_encrypt_pem_encoding(self, backend, options):
+        data = b"hello world\n"
+        cert, private_key = _load_rsa_cert_key()
+        builder = (
+            pkcs7.PKCS7EnvelopeBuilder().set_data(data).add_recipient(cert)
+        )
+        enveloped = builder.encrypt(serialization.Encoding.PEM, options)
+        decrypted_bytes = test_support.pkcs7_decrypt(
+            serialization.Encoding.PEM,
+            enveloped,
+            private_key,
+            cert,
+            options,
+        )
+        # New lines are canonicalized to '\r\n' when not using Binary
+        expected_data = (
+            data
+            if pkcs7.PKCS7Options.Binary in options
+            else data.replace(b"\n", b"\r\n")
+        )
+        assert decrypted_bytes == expected_data
+
+    def test_smime_encrypt_multiple_recipients(self, backend):
+        data = b"hello world\n"
+        cert, _ = _load_rsa_cert_key()
+        builder = (
+            pkcs7.PKCS7EnvelopeBuilder()
+            .set_data(data)
+            .add_recipient(cert)
+            .add_recipient(cert)
+        )
+        enveloped = builder.encrypt(serialization.Encoding.DER, [])
+        # cryptography CA (the recipient's Common Name)
+        common_name_bytes = (
+            b"\x0c\x0f\x63\x72\x79\x70\x74\x6f\x67\x72\x61"
+            b"\x70\x68\x79\x20\x43\x41"
+        )
+        assert enveloped.count(common_name_bytes) == 2
+
+
 @pytest.mark.supported(
     only_if=lambda backend: backend.pkcs7_supported(),
     skip_message="Requires OpenSSL with PKCS7 support",
@@ -822,5 +1141,30 @@ class TestPKCS7SerializeCerts:
 
         with pytest.raises(TypeError):
             pkcs7.serialize_certificates(
-                certs, "not an encoding"  # type: ignore[arg-type]
+                certs,
+                "not an encoding",  # type: ignore[arg-type]
             )
+
+
+@pytest.mark.supported(
+    only_if=lambda backend: not backend.pkcs7_supported(),
+    skip_message="Requires OpenSSL without PKCS7 support (BoringSSL)",
+)
+class TestPKCS7Unsupported:
+    def test_pkcs7_functions_unsupported(self):
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_SERIALIZATION):
+            pkcs7.load_der_pkcs7_certificates(b"nonsense")
+
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_SERIALIZATION):
+            pkcs7.load_pem_pkcs7_certificates(b"nonsense")
+
+
+@pytest.mark.supported(
+    only_if=lambda backend: backend.pkcs7_supported()
+    and not backend.rsa_encryption_supported(padding.PKCS1v15()),
+    skip_message="Requires OpenSSL with no PKCS1 v1.5 padding support",
+)
+class TestPKCS7EnvelopeBuilderUnsupported:
+    def test_envelope_builder_unsupported(self, backend):
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_PADDING):
+            pkcs7.PKCS7EnvelopeBuilder()
