@@ -38,6 +38,9 @@ from ...doubles import DummyKeySerializationEncryption
 from ...utils import load_vectors_from_file, raises_unsupported_algorithm
 from .fixtures_rsa import RSA_KEY_2048
 from .test_ec import _skip_curve_unsupported
+from .test_rsa import rsa_key_2048
+
+__all__ = ["rsa_key_2048"]
 
 
 class TestOpenSSHSerialization:
@@ -52,6 +55,10 @@ class TestOpenSSHSerialization:
             ("ecdsa-nopsw.key.pub", "ecdsa-nopsw.key-cert.pub"),
             ("ed25519-psw.key.pub", None),
             ("ed25519-nopsw.key.pub", "ed25519-nopsw.key-cert.pub"),
+            ("sk-ecdsa-psw.key.pub", None),
+            ("sk-ecdsa-nopsw.key.pub", None),
+            ("sk-ed25519-psw.key.pub", None),
+            ("sk-ed25519-nopsw.key.pub", None),
         ],
     )
     def test_load_ssh_public_key(self, key_file, cert_file, backend):
@@ -77,10 +84,14 @@ class TestOpenSSHSerialization:
                 )
         else:
             public_key = load_ssh_public_key(pub_data, backend)
-            assert (
-                public_key.public_bytes(Encoding.OpenSSH, PublicFormat.OpenSSH)
-                == nocomment_data
-            )
+            if not key_file.startswith("sk-"):
+                # SK keys do not round-trip
+                assert (
+                    public_key.public_bytes(
+                        Encoding.OpenSSH, PublicFormat.OpenSSH
+                    )
+                    == nocomment_data
+                )
 
         self.run_partial_pubkey(pub_data, backend)
 
@@ -387,7 +398,7 @@ class TestOpenSSHSerialization:
             b"\x04" * 65,
         ),
         priv_type=None,
-        priv_fields=(b"nistp256", b"\x04" * 65, b"\x7F" * 32),
+        priv_fields=(b"nistp256", b"\x04" * 65, b"\x7f" * 32),
         comment=b"comment",
         checkval1=b"1234",
         checkval2=b"1234",
@@ -401,12 +412,12 @@ class TestOpenSSHSerialization:
             priv_type = pub_type
 
         pub = ssh._FragList()
-        for elem in (pub_type,) + pub_fields:
+        for elem in (pub_type, *pub_fields):
             pub.put_sshstr(elem)
 
         secret = ssh._FragList([checkval1, checkval2])
         for i in range(nkeys):
-            for elem in (priv_type,) + priv_fields + (comment,):
+            for elem in (priv_type, *priv_fields, comment):
                 secret.put_sshstr(elem)
 
         if pad is None:
@@ -589,7 +600,9 @@ class TestOpenSSHSerialization:
                 Encoding.PEM, PrivateFormat.OpenSSH, NoEncryption()
             )
 
-    def test_serialize_ssh_private_key_errors(self, backend):
+    def test_serialize_ssh_private_key_errors(
+        self, rsa_key_2048: rsa.RSAPrivateKey, backend
+    ):
         # bad encoding
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
         with pytest.raises(ValueError):
@@ -615,6 +628,11 @@ class TestOpenSSHSerialization:
                 DummyKeySerializationEncryption(),
             )
 
+        with pytest.raises(ValueError):
+            rsa_key_2048.private_bytes(
+                Encoding.DER, PrivateFormat.OpenSSH, NoEncryption()
+            )
+
     @pytest.mark.supported(
         only_if=lambda backend: ssh._bcrypt_supported,
         skip_message="Requires that bcrypt exists",
@@ -636,34 +654,40 @@ class TestOpenSSHSerialization:
         ],
     )
     def test_serialize_ssh_private_key_with_password(
-        self, password, kdf_rounds, backend
+        self, password, kdf_rounds, rsa_key_2048: rsa.RSAPrivateKey, backend
     ):
-        original_key = ec.generate_private_key(ec.SECP256R1(), backend)
-        encoded_key_data = original_key.private_bytes(
-            Encoding.PEM,
-            PrivateFormat.OpenSSH,
-            (
-                PrivateFormat.OpenSSH.encryption_builder()
-                .kdf_rounds(kdf_rounds)
-                .build(password)
-            ),
-        )
+        for original_key in [
+            ec.generate_private_key(ec.SECP256R1(), backend),
+            rsa_key_2048,
+        ]:
+            assert isinstance(
+                original_key, (ec.EllipticCurvePrivateKey, rsa.RSAPrivateKey)
+            )
+            encoded_key_data = original_key.private_bytes(
+                Encoding.PEM,
+                PrivateFormat.OpenSSH,
+                (
+                    PrivateFormat.OpenSSH.encryption_builder()
+                    .kdf_rounds(kdf_rounds)
+                    .build(password)
+                ),
+            )
 
-        decoded_key = load_ssh_private_key(
-            data=encoded_key_data,
-            password=password,
-            backend=backend,
-        )
+            decoded_key = load_ssh_private_key(
+                data=encoded_key_data,
+                password=password,
+                backend=backend,
+            )
 
-        original_public_key = original_key.public_key().public_bytes(
-            Encoding.OpenSSH, PublicFormat.OpenSSH
-        )
+            original_public_key = original_key.public_key().public_bytes(
+                Encoding.OpenSSH, PublicFormat.OpenSSH
+            )
 
-        decoded_public_key = decoded_key.public_key().public_bytes(
-            Encoding.OpenSSH, PublicFormat.OpenSSH
-        )
+            decoded_public_key = decoded_key.public_key().public_bytes(
+                Encoding.OpenSSH, PublicFormat.OpenSSH
+            )
 
-        assert original_public_key == decoded_public_key
+            assert original_public_key == decoded_public_key
 
     @pytest.mark.supported(
         only_if=lambda backend: backend.dsa_supported(),
@@ -1159,31 +1183,6 @@ class TestSSHCertificate:
             b"permit-user-rc": b"",
         }
 
-    def test_loads_deprecated_invalid_encoding_cert(self, backend):
-        with pytest.warns(utils.DeprecatedIn41):
-            cert = load_ssh_public_identity(
-                b"ecdsa-sha2-nistp256-cert-v01@openssh.com AAAAKGVjZHNhLXNoYT"
-                b"ItbmlzdHAyNTYtY2VydC12MDFAb3BlbnNzaC5jb20AAAAgXE7sJ+xDVVNCO"
-                b"cEvpZS+SXIbc0nJdny/KqVbnwHslMIAAAAIbmlzdHAyNTYAAABBBI/qcLq8"
-                b"iiErpAhOWRqdMkpFSCNv7TVUcXCIfAl01JXbe2MvS4V7lFtiyrBjLSV7Iyw"
-                b"3TrulrWLibjPzZvLwmQcAAAAAAAAAAAAAAAEAAAAAAAAAAAAAAAAAAAAA//"
-                b"////////8AAABUAAAADWZvcmNlLWNvbW1hbmQAAAAoZWNobyBhYWFhYWFhY"
-                b"WFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYQAAAA92ZXJpZnktcmVxdWly"
-                b"ZWQAAAAAAAAAEgAAAApwZXJtaXQtcHR5AAAAAAAAAAAAAABoAAAAE2VjZHN"
-                b"hLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBI/qcLq8iiErpAhOWR"
-                b"qdMkpFSCNv7TVUcXCIfAl01JXbe2MvS4V7lFtiyrBjLSV7Iyw3TrulrWLib"
-                b"jPzZvLwmQcAAABlAAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAABKAAAAIQCi"
-                b"eCsIhGKrZdkE1+zY5EBucrLzxFpwnm/onIT/6rapvQAAACEAuVQ1yQjlPKr"
-                b"kfsGfjeG+2umZrOS5Ycx85BQhYf0RgsA="
-            )
-        assert isinstance(cert, SSHCertificate)
-        cert.verify_cert_signature()
-        assert cert.extensions == {b"permit-pty": b""}
-        assert cert.critical_options == {
-            b"force-command": b"echo aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
-            b"verify-required": b"",
-        }
-
     @pytest.mark.parametrize(
         "filename",
         [
@@ -1510,11 +1509,13 @@ class TestSSHCertificateBuilder:
         builder = SSHCertificateBuilder()
         with pytest.raises(TypeError):
             builder.add_critical_option(
-                "not bytes", b"test"  # type: ignore[arg-type]
+                "not bytes",  # type: ignore[arg-type]
+                b"test",
             )
         with pytest.raises(TypeError):
             builder.add_critical_option(
-                b"test", object()  # type: ignore[arg-type]
+                b"test",
+                object(),  # type: ignore[arg-type]
             )
         builder = builder.add_critical_option(b"test", b"test")
         with pytest.raises(ValueError):
@@ -1524,7 +1525,8 @@ class TestSSHCertificateBuilder:
         builder = SSHCertificateBuilder()
         with pytest.raises(TypeError):
             builder.add_extension(
-                "not bytes", b"test"  # type: ignore[arg-type]
+                "not bytes",  # type: ignore[arg-type]
+                b"test",
             )
         with pytest.raises(TypeError):
             builder.add_extension(b"test", object())  # type: ignore[arg-type]
@@ -1806,3 +1808,20 @@ class TestSSHCertificateBuilder:
             b"t8yRa8IRbxvOyA9TZYDGG1dRE3DiR0fuudU20v6vqfTd1gx0S5QyEdECXLl9ZI3"
             b"AwZgc="
         )
+
+
+class TestSSHSK:
+    @staticmethod
+    def ssh_str(application):
+        data = (
+            len(application).to_bytes(length=4, byteorder="big")
+            + application.encode()
+        )
+        return memoryview(data)
+
+    def test_load_application(self):
+        ssh.load_application(self.ssh_str("ssh:test"))
+
+    def test_load_application_valueerror(self):
+        with pytest.raises(ValueError):
+            ssh.load_application(self.ssh_str("hss:test"))
