@@ -6,10 +6,11 @@
 import base64
 import datetime
 import os
+from typing import Optional
 
 import pytest
 
-from cryptography import x509
+from cryptography import utils, x509
 from cryptography.exceptions import UnsupportedAlgorithm
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec, ed448, ed25519, rsa
@@ -68,6 +69,35 @@ def _generate_root(private_key=None, algorithm=hashes.SHA256()):
     return cert, private_key
 
 
+def _check_ocsp_response_times(
+    ocsp_resp: ocsp.OCSPResponse,
+    this_update: datetime.datetime,
+    next_update: Optional[datetime.datetime],
+    revocation_time: Optional[datetime.datetime],
+) -> None:
+    with pytest.warns(utils.DeprecatedIn43):
+        assert ocsp_resp.this_update == this_update
+    assert ocsp_resp.this_update_utc == this_update.replace(
+        tzinfo=datetime.timezone.utc
+    )
+
+    with pytest.warns(utils.DeprecatedIn43):
+        assert ocsp_resp.next_update == next_update
+    assert ocsp_resp.next_update_utc == (
+        next_update.replace(tzinfo=datetime.timezone.utc)
+        if next_update is not None
+        else None
+    )
+
+    with pytest.warns(utils.DeprecatedIn43):
+        assert ocsp_resp.revocation_time == revocation_time
+    assert ocsp_resp.revocation_time_utc == (
+        revocation_time.replace(tzinfo=datetime.timezone.utc)
+        if revocation_time is not None
+        else None
+    )
+
+
 class TestOCSPRequest:
     def test_bad_request(self):
         with pytest.raises(ValueError):
@@ -78,11 +108,12 @@ class TestOCSPRequest:
             os.path.join("x509", "ocsp", "req-sha1.der"),
             ocsp.load_der_ocsp_request,
         )
+        assert isinstance(req, ocsp.OCSPRequest)
         assert req.issuer_name_hash == (
-            b"8\xcaF\x8c\x07D\x8d\xf4\x81\x96" b"\xc7mmLpQ\x9e`\xa7\xbd"
+            b"8\xcaF\x8c\x07D\x8d\xf4\x81\x96\xc7mmLpQ\x9e`\xa7\xbd"
         )
         assert req.issuer_key_hash == (
-            b"yu\xbb\x84:\xcb,\xdez\t\xbe1" b"\x1bC\xbc\x1c*MSX"
+            b"yu\xbb\x84:\xcb,\xdez\t\xbe1\x1bC\xbc\x1c*MSX"
         )
         assert isinstance(req.hash_algorithm, hashes.SHA1)
         assert req.serial_number == int(
@@ -203,7 +234,10 @@ class TestOCSPRequestBuilder:
         builder = ocsp.OCSPRequestBuilder()
         with pytest.raises(ValueError):
             builder.add_certificate_by_hash(
-                b"0" * 20, b"0" * 20, 1, "notahash"  # type:ignore[arg-type]
+                b"0" * 20,
+                b"0" * 20,
+                1,
+                "notahash",  # type:ignore[arg-type]
             )
         with pytest.raises(ValueError):
             builder.add_certificate_by_hash(
@@ -304,7 +338,7 @@ class TestOCSPRequestBuilder:
         assert req.extensions[0].critical is critical
 
     def test_add_cert_by_hash(self):
-        cert, issuer = _cert_and_issuer()
+        cert, _ = _cert_and_issuer()
         builder = ocsp.OCSPRequestBuilder()
         h = hashes.Hash(hashes.SHA1())
         h.update(cert.issuer.public_bytes())
@@ -355,7 +389,9 @@ class TestOCSPResponseBuilder:
 
     def test_invalid_add_response(self):
         cert, issuer = _cert_and_issuer()
-        time = datetime.datetime.utcnow()
+        time = datetime.datetime.now(datetime.timezone.utc).replace(
+            tzinfo=None
+        )
         reason = x509.ReasonFlags.cessation_of_operation
         builder = ocsp.OCSPResponseBuilder()
         with pytest.raises(TypeError):
@@ -514,13 +550,18 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         with pytest.raises(TypeError):
             builder.add_extension(
-                "notanextension", True  # type: ignore[arg-type]
+                "notanextension",  # type: ignore[arg-type]
+                True,
             )
 
     def test_unsupported_extension(self):
         root_cert, private_key = _generate_root()
         cert, issuer = _cert_and_issuer()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
 
@@ -555,7 +596,9 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         _, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now().replace(tzinfo=None).replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.add_response(
@@ -575,7 +618,9 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now().replace(tzinfo=None).replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.responder_id(
@@ -597,7 +642,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.responder_id(
@@ -615,16 +664,26 @@ class TestOCSPResponseBuilder:
         resp = builder.sign(private_key, hashes.SHA256())
         assert resp.responder_name == root_cert.subject
         assert resp.responder_key_hash is None
-        assert (current_time - resp.produced_at).total_seconds() < 10
+        with pytest.warns(utils.DeprecatedIn43):
+            assert (current_time - resp.produced_at).total_seconds() < 10
+        assert (
+            current_time.replace(tzinfo=datetime.timezone.utc)
+            - resp.produced_at_utc
+        ).total_seconds() < 10
         assert (
             resp.signature_algorithm_oid
             == x509.SignatureAlgorithmOID.ECDSA_WITH_SHA256
         )
         assert resp.certificate_status == ocsp.OCSPCertStatus.GOOD
-        assert resp.revocation_time is None
         assert resp.revocation_reason is None
-        assert resp.this_update == this_update
-        assert resp.next_update == next_update
+
+        _check_ocsp_response_times(
+            resp,
+            this_update=this_update,
+            next_update=next_update,
+            revocation_time=None,
+        )
+
         private_key.public_key().verify(
             resp.signature, resp.tbs_response_bytes, ec.ECDSA(hashes.SHA256())
         )
@@ -633,7 +692,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         revoked_date = this_update - datetime.timedelta(days=300)
@@ -651,10 +714,13 @@ class TestOCSPResponseBuilder:
         )
         resp = builder.sign(private_key, hashes.SHA256())
         assert resp.certificate_status == ocsp.OCSPCertStatus.REVOKED
-        assert resp.revocation_time == revoked_date
         assert resp.revocation_reason is None
-        assert resp.this_update == this_update
-        assert resp.next_update == next_update
+        _check_ocsp_response_times(
+            resp,
+            this_update=this_update,
+            next_update=next_update,
+            revocation_time=revoked_date,
+        )
         private_key.public_key().verify(
             resp.signature, resp.tbs_response_bytes, ec.ECDSA(hashes.SHA256())
         )
@@ -663,7 +729,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.responder_id(
@@ -680,8 +750,12 @@ class TestOCSPResponseBuilder:
         )
         resp = builder.sign(private_key, hashes.SHA384())
         assert resp.certificate_status == ocsp.OCSPCertStatus.UNKNOWN
-        assert resp.this_update == this_update
-        assert resp.next_update == next_update
+        _check_ocsp_response_times(
+            resp,
+            this_update=this_update,
+            next_update=next_update,
+            revocation_time=None,
+        )
         private_key.public_key().verify(
             resp.signature, resp.tbs_response_bytes, ec.ECDSA(hashes.SHA384())
         )
@@ -690,7 +764,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = (
@@ -714,7 +792,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         revoked_date = this_update - datetime.timedelta(days=300)
         builder = builder.responder_id(
@@ -731,10 +813,13 @@ class TestOCSPResponseBuilder:
         )
         resp = builder.sign(private_key, hashes.SHA256())
         assert resp.certificate_status == ocsp.OCSPCertStatus.REVOKED
-        assert resp.revocation_time == revoked_date
         assert resp.revocation_reason is None
-        assert resp.this_update == this_update
-        assert resp.next_update is None
+        _check_ocsp_response_times(
+            resp,
+            this_update=this_update,
+            next_update=None,
+            revocation_time=revoked_date,
+        )
         private_key.public_key().verify(
             resp.signature, resp.tbs_response_bytes, ec.ECDSA(hashes.SHA256())
         )
@@ -743,7 +828,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         revoked_date = this_update - datetime.timedelta(days=300)
@@ -761,10 +850,13 @@ class TestOCSPResponseBuilder:
         )
         resp = builder.sign(private_key, hashes.SHA256())
         assert resp.certificate_status == ocsp.OCSPCertStatus.REVOKED
-        assert resp.revocation_time == revoked_date
         assert resp.revocation_reason is x509.ReasonFlags.key_compromise
-        assert resp.this_update == this_update
-        assert resp.next_update == next_update
+        _check_ocsp_response_times(
+            resp,
+            this_update=this_update,
+            next_update=next_update,
+            revocation_time=revoked_date,
+        )
         private_key.public_key().verify(
             resp.signature, resp.tbs_response_bytes, ec.ECDSA(hashes.SHA256())
         )
@@ -773,7 +865,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.responder_id(
@@ -800,8 +896,12 @@ class TestOCSPResponseBuilder:
     def test_invalid_sign_responder_cert_does_not_match_private_key(self):
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
-        root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        root_cert, _ = _generate_root()
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.responder_id(
@@ -826,7 +926,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = (
@@ -882,7 +986,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, _ = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.responder_id(
@@ -910,7 +1018,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.responder_id(
@@ -933,7 +1045,11 @@ class TestOCSPResponseBuilder:
         builder = ocsp.OCSPResponseBuilder()
         cert, issuer = _cert_and_issuer()
         root_cert, private_key = _generate_root()
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         builder = builder.responder_id(
@@ -1058,6 +1174,7 @@ class TestOCSPResponse:
             os.path.join("x509", "letsencryptx3.pem"),
             x509.load_pem_x509_certificate,
         )
+        assert isinstance(resp, ocsp.OCSPResponse)
         assert resp.response_status == ocsp.OCSPResponseStatus.SUCCESSFUL
         assert (
             resp.signature_algorithm_oid
@@ -1090,12 +1207,19 @@ class TestOCSPResponse:
         assert resp.certificates == []
         assert resp.responder_key_hash is None
         assert resp.responder_name == issuer.subject
-        assert resp.produced_at == datetime.datetime(2018, 8, 30, 11, 15)
+        with pytest.warns(utils.DeprecatedIn43):
+            assert resp.produced_at == datetime.datetime(2018, 8, 30, 11, 15)
+        assert resp.produced_at_utc == datetime.datetime(
+            2018, 8, 30, 11, 15, tzinfo=datetime.timezone.utc
+        )
         assert resp.certificate_status == ocsp.OCSPCertStatus.GOOD
-        assert resp.revocation_time is None
         assert resp.revocation_reason is None
-        assert resp.this_update == datetime.datetime(2018, 8, 30, 11, 0)
-        assert resp.next_update == datetime.datetime(2018, 9, 6, 11, 0)
+        _check_ocsp_response_times(
+            resp,
+            this_update=datetime.datetime(2018, 8, 30, 11, 0),
+            next_update=datetime.datetime(2018, 9, 6, 11, 0),
+            revocation_time=None,
+        )
         assert resp.issuer_key_hash == (
             b"\xa8Jjc\x04}\xdd\xba\xe6\xd19\xb7\xa6Ee\xef\xf3\xa8\xec\xa1"
         )
@@ -1115,6 +1239,7 @@ class TestOCSPResponse:
         with pytest.raises(ValueError):
             resp.serial_number
 
+        assert isinstance(next(resp.responses), ocsp.OCSPSingleResponse)
         assert len(list(resp.responses)) == 20
 
     def test_multi_valued_responses(self):
@@ -1150,9 +1275,20 @@ class TestOCSPResponse:
                 )
 
                 assert elem.certificate_status == ocsp.OCSPCertStatus.GOOD
-
-                assert elem.this_update == datetime.datetime(2020, 2, 22, 0, 0)
-                assert elem.next_update == datetime.datetime(2020, 2, 29, 1, 0)
+                with pytest.warns(utils.DeprecatedIn43):
+                    assert elem.this_update == datetime.datetime(
+                        2020, 2, 22, 0, 0
+                    )
+                assert elem.this_update_utc == datetime.datetime(
+                    2020, 2, 22, 0, 0, tzinfo=datetime.timezone.utc
+                )
+                with pytest.warns(utils.DeprecatedIn43):
+                    assert elem.next_update == datetime.datetime(
+                        2020, 2, 29, 1, 0
+                    )
+                assert elem.next_update_utc == datetime.datetime(
+                    2020, 2, 29, 1, 0, tzinfo=datetime.timezone.utc
+                )
             elif req_revoked.serial_number == serial:
                 assert elem.certificate_status == ocsp.OCSPCertStatus.REVOKED
 
@@ -1160,8 +1296,12 @@ class TestOCSPResponse:
                     elem.revocation_reason
                     == x509.ReasonFlags.cessation_of_operation
                 )
-                assert elem.revocation_time == datetime.datetime(
-                    2018, 5, 30, 14, 1, 39
+                with pytest.warns(utils.DeprecatedIn43):
+                    assert elem.revocation_time == datetime.datetime(
+                        2018, 5, 30, 14, 1, 39
+                    )
+                assert elem.revocation_time_utc == datetime.datetime(
+                    2018, 5, 30, 14, 1, 39, tzinfo=datetime.timezone.utc
                 )
 
     def test_load_unauthorized(self):
@@ -1184,18 +1324,26 @@ class TestOCSPResponse:
             resp.responder_key_hash
         with pytest.raises(ValueError):
             resp.responder_name
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError), pytest.warns(utils.DeprecatedIn43):
             resp.produced_at
         with pytest.raises(ValueError):
-            resp.certificate_status
+            resp.produced_at_utc
         with pytest.raises(ValueError):
+            resp.certificate_status
+        with pytest.raises(ValueError), pytest.warns(utils.DeprecatedIn43):
             resp.revocation_time
         with pytest.raises(ValueError):
-            resp.revocation_reason
+            resp.revocation_time_utc
         with pytest.raises(ValueError):
+            resp.revocation_reason
+        with pytest.raises(ValueError), pytest.warns(utils.DeprecatedIn43):
             resp.this_update
         with pytest.raises(ValueError):
+            resp.this_update_utc
+        with pytest.raises(ValueError), pytest.warns(utils.DeprecatedIn43):
             resp.next_update
+        with pytest.raises(ValueError):
+            resp.next_update_utc
         with pytest.raises(ValueError):
             resp.issuer_key_hash
         with pytest.raises(ValueError):
@@ -1213,8 +1361,12 @@ class TestOCSPResponse:
             ocsp.load_der_ocsp_response,
         )
         assert resp.certificate_status == ocsp.OCSPCertStatus.REVOKED
-        assert resp.revocation_time == datetime.datetime(
-            2016, 9, 2, 21, 28, 48
+        with pytest.warns(utils.DeprecatedIn43):
+            assert resp.revocation_time == datetime.datetime(
+                2016, 9, 2, 21, 28, 48
+            )
+        assert resp.revocation_time_utc == datetime.datetime(
+            2016, 9, 2, 21, 28, 48, tzinfo=datetime.timezone.utc
         )
         assert resp.revocation_reason is None
 
@@ -1269,7 +1421,9 @@ class TestOCSPResponse:
             ocsp.load_der_ocsp_response,
         )
         assert resp.serial_number == 16160
-        assert resp.next_update is None
+        with pytest.warns(utils.DeprecatedIn43):
+            assert resp.next_update is None
+        assert resp.next_update_utc is None
 
     def test_response_extensions(self):
         resp = _load_data(
@@ -1380,7 +1534,11 @@ class TestOCSPEdDSA:
         cert, issuer = _cert_and_issuer()
         private_key = ed25519.Ed25519PrivateKey.generate()
         root_cert, _ = _generate_root(private_key, None)
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         revoked_date = this_update - datetime.timedelta(days=300)
@@ -1408,7 +1566,11 @@ class TestOCSPEdDSA:
         cert, issuer = _cert_and_issuer()
         private_key = ed25519.Ed25519PrivateKey.generate()
         root_cert, _ = _generate_root(private_key, None)
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         revoked_date = this_update - datetime.timedelta(days=300)
@@ -1426,10 +1588,13 @@ class TestOCSPEdDSA:
         )
         resp = builder.sign(private_key, None)
         assert resp.certificate_status == ocsp.OCSPCertStatus.REVOKED
-        assert resp.revocation_time == revoked_date
         assert resp.revocation_reason is x509.ReasonFlags.key_compromise
-        assert resp.this_update == this_update
-        assert resp.next_update == next_update
+        _check_ocsp_response_times(
+            resp,
+            this_update=this_update,
+            next_update=next_update,
+            revocation_time=revoked_date,
+        )
         assert resp.signature_hash_algorithm is None
         assert (
             resp.signature_algorithm_oid == x509.SignatureAlgorithmOID.ED25519
@@ -1447,7 +1612,11 @@ class TestOCSPEdDSA:
         cert, issuer = _cert_and_issuer()
         private_key = ed448.Ed448PrivateKey.generate()
         root_cert, _ = _generate_root(private_key, None)
-        current_time = datetime.datetime.utcnow().replace(microsecond=0)
+        current_time = (
+            datetime.datetime.now(datetime.timezone.utc)
+            .replace(tzinfo=None)
+            .replace(microsecond=0)
+        )
         this_update = current_time - datetime.timedelta(days=1)
         next_update = this_update + datetime.timedelta(days=7)
         revoked_date = this_update - datetime.timedelta(days=300)
@@ -1465,10 +1634,13 @@ class TestOCSPEdDSA:
         )
         resp = builder.sign(private_key, None)
         assert resp.certificate_status == ocsp.OCSPCertStatus.REVOKED
-        assert resp.revocation_time == revoked_date
         assert resp.revocation_reason is x509.ReasonFlags.key_compromise
-        assert resp.this_update == this_update
-        assert resp.next_update == next_update
+        _check_ocsp_response_times(
+            resp,
+            this_update=this_update,
+            next_update=next_update,
+            revocation_time=revoked_date,
+        )
         assert resp.signature_hash_algorithm is None
         assert resp.signature_algorithm_oid == x509.SignatureAlgorithmOID.ED448
         private_key.public_key().verify(
