@@ -7,10 +7,16 @@ import datetime
 
 import pytest
 
-from cryptography import x509
+from cryptography import utils, x509
 from cryptography.exceptions import UnsupportedAlgorithm
 from cryptography.hazmat.primitives import hashes
-from cryptography.hazmat.primitives.asymmetric import ec, ed448, ed25519, rsa
+from cryptography.hazmat.primitives.asymmetric import (
+    ec,
+    ed448,
+    ed25519,
+    padding,
+    rsa,
+)
 from cryptography.x509.oid import (
     AuthorityInformationAccessOID,
     NameOID,
@@ -65,7 +71,11 @@ class TestCertificateRevocationListBuilder:
         )
 
         crl = builder.sign(private_key, hashes.SHA256(), backend)
-        assert crl.last_update == utc_last
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl.last_update == utc_last
+        assert crl.last_update_utc == utc_last.replace(
+            tzinfo=datetime.timezone.utc
+        )
 
     def test_last_update_invalid(self):
         builder = x509.CertificateRevocationListBuilder()
@@ -106,7 +116,11 @@ class TestCertificateRevocationListBuilder:
         )
 
         crl = builder.sign(private_key, hashes.SHA256(), backend)
-        assert crl.next_update == utc_next
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl.next_update == utc_next
+        assert crl.next_update_utc == utc_next.replace(
+            tzinfo=datetime.timezone.utc
+        )
 
     def test_next_update_invalid(self):
         builder = x509.CertificateRevocationListBuilder()
@@ -196,6 +210,38 @@ class TestCertificateRevocationListBuilder:
         with pytest.raises(ValueError):
             builder.sign(private_key, hashes.SHA256(), backend)
 
+    def test_sign_invalid_padding(
+        self, rsa_key_2048: rsa.RSAPrivateKey, backend
+    ):
+        last_update = datetime.datetime(2002, 1, 1, 12, 1)
+        next_update = datetime.datetime(2030, 1, 1, 12, 1)
+        builder = (
+            x509.CertificateRevocationListBuilder()
+            .issuer_name(
+                x509.Name(
+                    [
+                        x509.NameAttribute(
+                            NameOID.COMMON_NAME, "cryptography.io CA"
+                        )
+                    ]
+                )
+            )
+            .last_update(last_update)
+            .next_update(next_update)
+        )
+
+        with pytest.raises(TypeError):
+            builder.sign(
+                rsa_key_2048,
+                hashes.SHA256(),
+                rsa_padding=b"notapadding",  # type: ignore[arg-type]
+            )
+        eckey = ec.generate_private_key(ec.SECP256R1())
+        with pytest.raises(TypeError):
+            builder.sign(
+                eckey, hashes.SHA256(), rsa_padding=padding.PKCS1v15()
+            )
+
     def test_sign_empty_list(self, rsa_key_2048: rsa.RSAPrivateKey, backend):
         private_key = rsa_key_2048
         last_update = datetime.datetime(2002, 1, 1, 12, 1)
@@ -217,8 +263,49 @@ class TestCertificateRevocationListBuilder:
 
         crl = builder.sign(private_key, hashes.SHA256(), backend)
         assert len(crl) == 0
-        assert crl.last_update == last_update
-        assert crl.next_update == next_update
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl.last_update == last_update
+            assert crl.next_update == next_update
+        assert crl.last_update_utc == last_update.replace(
+            tzinfo=datetime.timezone.utc
+        )
+        assert crl.next_update_utc == next_update.replace(
+            tzinfo=datetime.timezone.utc
+        )
+
+    def test_sign_pss(self, rsa_key_2048: rsa.RSAPrivateKey, backend):
+        private_key = rsa_key_2048
+        last_update = datetime.datetime(2002, 1, 1, 12, 1)
+        next_update = datetime.datetime(2030, 1, 1, 12, 1)
+        builder = (
+            x509.CertificateRevocationListBuilder()
+            .issuer_name(
+                x509.Name(
+                    [
+                        x509.NameAttribute(
+                            NameOID.COMMON_NAME, "cryptography.io CA"
+                        )
+                    ]
+                )
+            )
+            .last_update(last_update)
+            .next_update(next_update)
+        )
+
+        pss = padding.PSS(
+            mgf=padding.MGF1(hashes.SHA256()),
+            salt_length=padding.PSS.DIGEST_LENGTH,
+        )
+        crl = builder.sign(private_key, hashes.SHA256(), rsa_padding=pss)
+        assert len(crl) == 0
+        assert isinstance(crl.signature_algorithm_parameters, padding.PSS)
+        assert crl.signature_algorithm_parameters._salt_length == 32
+        private_key.public_key().verify(
+            crl.signature,
+            crl.tbs_certlist_bytes,
+            crl.signature_algorithm_parameters,
+            hashes.SHA256(),
+        )
 
     @pytest.mark.parametrize(
         "extension",
@@ -402,7 +489,11 @@ class TestCertificateRevocationListBuilder:
             .add_revoked_certificate(
                 x509.RevokedCertificateBuilder()
                 .serial_number(1234)
-                .revocation_date(datetime.datetime.utcnow())
+                .revocation_date(
+                    datetime.datetime.now(datetime.timezone.utc).replace(
+                        tzinfo=None
+                    )
+                )
                 .add_extension(DummyExtension(), critical=False)
                 .build()
             )
@@ -457,7 +548,9 @@ class TestCertificateRevocationListBuilder:
 
         with pytest.raises(TypeError):
             builder.sign(
-                private_key, object(), backend  # type: ignore[arg-type]
+                private_key,
+                object(),  # type: ignore[arg-type]
+                backend,
             )
 
     @pytest.mark.supported(
@@ -570,7 +663,9 @@ class TestCertificateRevocationListBuilder:
             == ian
         )
         assert crl[0].serial_number == revoked_cert0.serial_number
-        assert crl[0].revocation_date == revoked_cert0.revocation_date
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl[0].revocation_date == revoked_cert0.revocation_date
+        assert crl[0].revocation_date_utc == revoked_cert0.revocation_date_utc
         assert len(crl[0].extensions) == 1
         ext = crl[0].extensions.get_extension_for_class(x509.InvalidityDate)
         assert ext.critical is False
@@ -619,7 +714,9 @@ class TestCertificateRevocationListBuilder:
             == ian
         )
         assert crl[0].serial_number == revoked_cert0.serial_number
-        assert crl[0].revocation_date == revoked_cert0.revocation_date
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl[0].revocation_date == revoked_cert0.revocation_date
+        assert crl[0].revocation_date_utc == revoked_cert0.revocation_date_utc
         assert len(crl[0].extensions) == 1
         ext = crl[0].extensions.get_extension_for_class(x509.InvalidityDate)
         assert ext.critical is False
@@ -673,7 +770,9 @@ class TestCertificateRevocationListBuilder:
             == ian
         )
         assert crl[0].serial_number == revoked_cert0.serial_number
-        assert crl[0].revocation_date == revoked_cert0.revocation_date
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl[0].revocation_date == revoked_cert0.revocation_date
+        assert crl[0].revocation_date_utc == revoked_cert0.revocation_date_utc
         assert len(crl[0].extensions) == 1
         ext = crl[0].extensions.get_extension_for_class(x509.InvalidityDate)
         assert ext.critical is False
@@ -727,7 +826,9 @@ class TestCertificateRevocationListBuilder:
             == ian
         )
         assert crl[0].serial_number == revoked_cert0.serial_number
-        assert crl[0].revocation_date == revoked_cert0.revocation_date
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl[0].revocation_date == revoked_cert0.revocation_date
+        assert crl[0].revocation_date_utc == revoked_cert0.revocation_date_utc
         assert len(crl[0].extensions) == 1
         ext = crl[0].extensions.get_extension_for_class(x509.InvalidityDate)
         assert ext.critical is False
@@ -754,7 +855,9 @@ class TestCertificateRevocationListBuilder:
 
         with pytest.raises(UnsupportedAlgorithm):
             builder.sign(
-                private_key, hashes.MD5(), backend  # type: ignore[arg-type]
+                private_key,
+                hashes.MD5(),  # type: ignore[arg-type]
+                backend,
             )
 
     def test_ec_key_sign_md5(self, backend):
@@ -779,7 +882,9 @@ class TestCertificateRevocationListBuilder:
 
         with pytest.raises(UnsupportedAlgorithm):
             builder.sign(
-                private_key, hashes.MD5(), backend  # type: ignore[arg-type]
+                private_key,
+                hashes.MD5(),  # type: ignore[arg-type]
+                backend,
             )
 
     def test_sign_with_revoked_certificates(
@@ -835,13 +940,24 @@ class TestCertificateRevocationListBuilder:
 
         crl = builder.sign(private_key, hashes.SHA256(), backend)
         assert len(crl) == 3
-        assert crl.last_update == last_update
-        assert crl.next_update == next_update
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl.last_update == last_update
+            assert crl.next_update == next_update
+        assert crl.last_update_utc == last_update.replace(
+            tzinfo=datetime.timezone.utc
+        )
+        assert crl.next_update_utc == next_update.replace(
+            tzinfo=datetime.timezone.utc
+        )
         assert crl[0].serial_number == revoked_cert0.serial_number
-        assert crl[0].revocation_date == revoked_cert0.revocation_date
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl[0].revocation_date == revoked_cert0.revocation_date
+        assert crl[0].revocation_date_utc == revoked_cert0.revocation_date_utc
         assert len(crl[0].extensions) == 0
         assert crl[1].serial_number == revoked_cert1.serial_number
-        assert crl[1].revocation_date == revoked_cert1.revocation_date
+        with pytest.warns(utils.DeprecatedIn42):
+            assert crl[1].revocation_date == revoked_cert1.revocation_date
+        assert crl[1].revocation_date_utc == revoked_cert1.revocation_date_utc
         assert len(crl[1].extensions) == 2
         ext = crl[1].extensions.get_extension_for_class(x509.InvalidityDate)
         assert ext.critical is False
